/*

  COPYRIGHT
  --------
  
    Copyright Maurizio Colucci -  9 nov 2003.
  
  COPYING
  -------
  
    This file is part of segusoLand.

    segusoLand is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    segusoLand is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with segusoLand; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  ABOUT THE AUTHOR
  ----------------

  You can contact me at seguso.forever@tin.it

  I live in Pescara, Italy.

  Job offers inside and outside Italy are very welcome :-)

 QUICK INTRO TO THE CODE
 -----------------------

*  If you are looking for the main() procedure, it is 
   run/1, in segusoLand.pl. Actually, run/1 is called by

      int main(void)

   which is written in C++, and located in this file.


*/



#include <SWI-Prolog.h>  
#include <string>
#include <map>
#include <set>
#include <vector>
#include <list>
#include <iostream>
#include <gtkmm.h>
#include <boost/shared_ptr.hpp>
using namespace boost;
//#include<libgen.h> // for dirname. Do not include here, because it imports a wrong version of dirname!
#include <sys/stat.h> // for stat()
using namespace Gtk;
using namespace Glib;
using namespace std;


const int gScrollButtonHeight = 36;

void throwPrologException(string s){
	cout << s << endl;
}

// REGION the global gtk toolbtips object. making it a shared_ptr makes it automatically deleted at shutdown
shared_ptr<Tooltips> gTooltips;
// END

// since you cannot store prolog term_t directly, because they are
// invalidated as soon as PL_succeed or PL_fail are called, I had to
// create a class SegTerm, which allows me to store a copy of a term_t
// which is not destroyed. SegTerm can translate itself to a term_t on
// demand.
class SegTerm{
public:
	SegTerm(term_t aPrologTerm){
		// make a copy of aPrologTerm and put it into
		// mRecordedTerm
		mTermHandle = PL_record(aPrologTerm);
	}
	/// copy constructor. It is important because if I were to
	/// leave the default bitwise copy, the two SegTerm would
	/// share the same mTermHandle. When one SegTerm is destroyed,
	/// it would call PL_erase, and the other SegTerm would
	/// contain an invalid mTermHandle.
	SegTerm(const SegTerm& t) {
		mTermHandle = PL_record(t.ToPrologTerm());
	}
	const term_t ToPrologTerm() const{
		term_t lRet = PL_new_term_ref();
		PL_recorded(mTermHandle, lRet/*out*/);
		return lRet;
	}
	ustring ToString() const{
		term_t lRet = PL_new_term_ref();
		PL_recorded(mTermHandle, lRet/*out*/);
		return termToString(lRet);
	}
	virtual ~SegTerm(){
		PL_erase(mTermHandle);
	}
	/// needed to use map<SegTerm, _>
	bool operator<(const SegTerm& s2) const{
		int lComparison= PL_compare(ToPrologTerm(), s2.ToPrologTerm());
		return lComparison<0;}

	/// if you only need to translate from term_t to string, use
	/// this. No need to instantiate a SegTerm
	static ustring termToString(term_t t) ;
private:
	record_t mTermHandle;

};

enum SearchState{
	ADDING_CHARS,
	SKIPPING_TO_NEXT_NONSEPARATOR};
std::set<ustring> gSplitStringIntoWords(ustring aString){
	if (!aString.validate()){
		cout << "gSplitStringIntoWords: invalid string (not utf) : " << aString <<endl;
		throw string();}
	std::set<ustring> lCurrentWordList;
	ustring lCurrentWord = "";
	SearchState lSearchState = ADDING_CHARS;
	for(int i=0;   i!=aString.length();  i++){
		gunichar lCurrentChar = aString[i];
		if (lSearchState == ADDING_CHARS){
			if(!g_unichar_isalnum(lCurrentChar)){
				if(lCurrentWord.length() >2){
					lCurrentWordList.insert(lCurrentWord);}
				lCurrentWord = "";
				lSearchState = SKIPPING_TO_NEXT_NONSEPARATOR;}
			else{
				lCurrentWord += lCurrentChar;}}
		else{
			if(g_unichar_isalnum(lCurrentChar)){
				lCurrentWord += lCurrentChar;
				lSearchState = ADDING_CHARS;}}}
	if (lCurrentWord.length() >2){
		lCurrentWordList.insert(lCurrentWord);}
	return lCurrentWordList;}

// REGION gLatestEvent: written by the callbacks. Read by
// sleepUntilEventIsRaised/1.
std::list<SegTerm> gLatestEvents;
// END

ustring SegTerm::termToString(term_t t) 
{
	char * name;
	int lReturn = PL_get_chars(t, &name, CVT_ALL | CVT_WRITE);
	if (lReturn==FALSE){
		cout << "termToString: PL_get_chars failed" << endl; 
		throw string("PL_get_chars failed");
	}
	ustring lResult(name);
	// REGION slow
	//if (!lResult.validate()){
	//	cout << "termToString: not a valid UTF string : "<< lResult << endl;}
	// END

	//free(name); // don't do this!
	return lResult;
}

//REGION gPrologNameToGtkWidget
//
// the prolog programmer, when he creates widgets, assigns them a
// name, which can be a term such as 
//
//    hello(dear(Jack))
//
// , and henceforth refers to those widget by that name. So we must
// remember which widget is assigned to a given name. We use map for
// this purpose.
//
//  gPrologNameToGtkWidget is read by predicates written in C++, such
//  as widgetHide/1.
//
//  the Pointer is necessary in order to activate polimorphism. 
//
// note: changing Widget * to shared_ptr<Widget> would be useless
// here, because widgets are often deleted directly by gtk when their
// father is deleted; so the shared_ptr would point to an invalid
// widget.
map<SegTerm, Widget* > gPrologNameToGtkWidget; 
//END


void callPredicate(const string& aGoal){
	fid_t fid = PL_open_foreign_frame();
	term_t g = PL_new_term_ref();
	
	if (PL_chars_to_term(aGoal.c_str(), g)){
		PL_call(g, NULL);
	}
	else
		throw new string("cannot convert argument " + aGoal + " to term");
}

class MyScrollButtonBox;
class MyAdjustment : public Adjustment{
public:
	enum SCROLLING_STATE{
		I_AM_SCROLLING_DOWN,
		I_AM_NOT_SCROLLING,
		I_AM_SCROLLING_UP
	};
	MyAdjustment(string name) ;
	void TheScrolledWindowHasBeenHidden();
	void TheScrolledWindowHasBeenShown();


	void WheelMovedDown(){
		if (mScrollingState == I_AM_NOT_SCROLLING){
			mScrollingState = I_AM_SCROLLING_DOWN;
			mScrollCurrentSpeed = mScrollStartSpeed;}
		else if (mScrollingState == I_AM_SCROLLING_UP){
			mScrollingState = I_AM_NOT_SCROLLING;}
		else{
			mScrollCurrentSpeed += mScrollIncrement;}}
	void WheelMovedUp(){
		if (mScrollingState == I_AM_NOT_SCROLLING){
			mScrollingState = I_AM_SCROLLING_UP;
			mScrollCurrentSpeed = mScrollStartSpeed;}
		else if (mScrollingState == I_AM_SCROLLING_DOWN){
			mScrollingState = I_AM_NOT_SCROLLING;}
		else{
			mScrollCurrentSpeed += mScrollIncrement;}}
	void GoToTopOrBottomButtonClicked(bool aTop);
	void TheSmoothScrollButtonWasPressed(bool aUp){
		if (aUp)
			mScrollingState = I_AM_SCROLLING_UP;
		else 
			mScrollingState = I_AM_SCROLLING_DOWN;
		mScrollCurrentSpeed = mScrollStartSpeed;}
	void TheSmoothScrollButtonWasReleased(){
		mScrollingState = I_AM_NOT_SCROLLING;	}
	void SetScrollButtonBoxes(MyScrollButtonBox * aUp, 
				  MyScrollButtonBox* aDown) ;
	void SetScrollPositionToTheTop(){
		mScrollingState = I_AM_NOT_SCROLLING;
		set_value(get_lower());}
	void TheMouseLeftTheButtonArea(bool aUp);
	void TheMouseEnteredTheButtonArea(bool aUp);
	void ScrollPage(bool aUp);
	void SetKeepScrollingBoxesAlwaysVisible(bool aKeep);
protected:
	virtual void on_value_changed();
	virtual void on_changed();
private:
	void ShowOrHideTheScrollButtonsAfterHidingPolicyChanged();
	string mName;
	void Update();
        bool on_idle(){
		Update();
		return true; /* false would stop on_idle being called */}
	MyScrollButtonBox* mUpBox;
	MyScrollButtonBox* mDownBox;
	SCROLLING_STATE mScrollingState;

	static const double mScrollStartSpeed = 1.3;
	double mScrollCurrentSpeed;
	static const double mScrollIncrement = 1.0;
	class ScrollBoxesHidingPolicy;
	class ScrollBoxesHidingPolicyHide;
	class ScrollBoxesHidingPolicyDoNotHide;

        shared_ptr<ScrollBoxesHidingPolicy> mCurrentScrollBoxesHidingPolicy;
};

class MyAdjustment::ScrollBoxesHidingPolicy{
public:
	virtual ~ScrollBoxesHidingPolicy(){}
};
class MyAdjustment::ScrollBoxesHidingPolicyHide : public ScrollBoxesHidingPolicy{
public:
	// REGION those values only make sense when the policy is
	// "hide". So I put them in this class, and not in
	// MyAdjustment; so these values do not exist when the policy
	// is "do not hide", and I cannot access them by
	// mistake. Defensive programming.
	bool mTheMouseIsNotOnTheBottomButtonBox;
	bool mTheMouseIsNotOnTheTopButtonBox;
	// END
	ScrollBoxesHidingPolicyHide(){ 
		// REGION it is very important for these value to be true by default. Otherwise
		// on_value_changed() will not be able to hide scrollbuttons until the mouse enters
		// and exits the button area at least once
		mTheMouseIsNotOnTheTopButtonBox = true;
		mTheMouseIsNotOnTheBottomButtonBox = true;
		// END
	}
};

class MyAdjustment::ScrollBoxesHidingPolicyDoNotHide : public ScrollBoxesHidingPolicy{
};
	


// a ScrolledWindow which uses a MyAdjustment instead of a normal
// Adjustment, and informs MyAdjustment when the wheel is moved.
class MyScrolledWindow: public ScrolledWindow{
public:
	MyScrolledWindow(shared_ptr<MyAdjustment>  a) : ScrolledWindow(*new Adjustment(0,0,1), *a.get()){
		mMyAdjustment = a;
	}
protected:
	virtual bool on_scroll_event (GdkEventScroll* event){
		if (event->direction == GDK_SCROLL_DOWN)
			mMyAdjustment->WheelMovedDown();
		else
			mMyAdjustment->WheelMovedUp(); 
	}
private:
	shared_ptr<MyAdjustment> mMyAdjustment;
};

class GoToTopOrBottomButton : public Button{
public:
	GoToTopOrBottomButton(shared_ptr<MyAdjustment> aMyAdj, bool aUp){
		mMyAdjustment = aMyAdj;

		mArrow = manage(new Arrow(aUp?ARROW_UP: ARROW_DOWN, SHADOW_IN));
		//lArrow1->show();
		add(*mArrow);
		show();
		//set_relief(RELIEF_NONE);
		set_size_request(30, gScrollButtonHeight);

		mUp = aUp;
		gTooltips->set_tip(*this, string("Click here to quickly reach the ") + (aUp? "top" : "bottom") + " of the list");}
protected:
	virtual void on_clicked(){
		mMyAdjustment->GoToTopOrBottomButtonClicked(mUp);
		Button::on_clicked();}
	virtual bool on_enter_notify_event(GdkEventCrossing * c){
		mArrow->show();	
		return Button::on_enter_notify_event(c);}
	virtual bool on_leave_notify_event(GdkEventCrossing *c){
		mArrow->hide();	
		return Button::on_leave_notify_event(c);}
private:
	shared_ptr<MyAdjustment> mMyAdjustment;
	bool mUp;
	Arrow * mArrow;
};

class MySmoothScrollButton : public Button{
public:
	MySmoothScrollButton(shared_ptr<MyAdjustment> aMyAdj, bool aUp){
		mMyAdjustment = aMyAdj;
		mUp = aUp;
		mArrow = manage(new Arrow(aUp? ARROW_UP : ARROW_DOWN, SHADOW_IN));
		//lArrow->show();
		add(*mArrow);

		show();
		//set_relief(RELIEF_NONE);
		set_size_request(30, gScrollButtonHeight);

		gTooltips->set_tip(*this, string("Click and hold this button to smoothly scroll ") + (aUp? "upwards" : "downwards") + ". However, it is better to use the mouse wheel, since the wheel allows changing the scroll speed. Use this button only if you don't have a wheel mouse.");}
protected:
	virtual bool on_button_press_event(GdkEventButton* event){
		mMyAdjustment->TheSmoothScrollButtonWasPressed(mUp); 
		return Button::on_button_press_event(event);
	}
	virtual bool on_button_release_event(GdkEventButton* event){
		mMyAdjustment->TheSmoothScrollButtonWasReleased();
		return Button::on_button_release_event(event);
	}
	virtual bool on_enter_notify_event(GdkEventCrossing * c){
		mArrow->show();	
		return Button::on_enter_notify_event(c);}
	virtual bool on_leave_notify_event(GdkEventCrossing *c){
		mArrow->hide();	
		return Button::on_leave_notify_event(c);}
private:
	shared_ptr<MyAdjustment> mMyAdjustment;
	bool mUp;
	Arrow * mArrow;
};



class MyPageScrollButton : public Button{
public:
	MyPageScrollButton(shared_ptr<MyAdjustment> aMyAdj, bool aUp){
		mMyAdjustment = aMyAdj;

		VBox * lVBox = manage(new VBox());
		lVBox->show();
		add(*lVBox);
		
		mArrow = manage(new Arrow(aUp?ARROW_UP: ARROW_DOWN, SHADOW_IN));
		mArrow->show();
		lVBox->pack_start(*mArrow, true, true);

		show();

		set_size_request(gScrollButtonHeight, gScrollButtonHeight);

		mUp = aUp;
		gTooltips->set_tip(*this, "Click here to scroll this list one page at a time. If you need a finer scrolling, you can use the mouse wheel. If you don't have the wheel, please buy a new mouse. No, wait! Buy TWO instead, and send one to me.\nOr just use the small button on the left.");
	}
	void ShowTheArrow(){
		mArrow->show();
	}
	void HideTheArrow(){
		mArrow->hide();
	}
protected:
	virtual void on_clicked(){
		//cout<< "page scroll clicked\n";
		mMyAdjustment->ScrollPage(mUp);
		Button::on_clicked();
	}
private:
	shared_ptr<MyAdjustment> mMyAdjustment;
	bool mUp;
	Arrow * mArrow; // managed by gtk, so no shared_ptr
};

class MyScrollButtonBox : public EventBox{
public:
	MyScrollButtonBox(shared_ptr<MyAdjustment> aAdj, bool aUp);

	void YouHaveReachedTheScrollingLimitAndYouMustBecomeInsensitive(bool aReached, string callerName);
protected:
	virtual bool on_leave_notify_event(GdkEventCrossing *c){
		//cout << "on_leave_notify_event" <<endl;
		mMyAdjustment->TheMouseLeftTheButtonArea(mUp);	
		return EventBox::on_leave_notify_event(c);}
	virtual bool on_enter_notify_event(GdkEventCrossing *c){
		//cout << "on_enter_notify_event" <<endl;
		mMyAdjustment->TheMouseEnteredTheButtonArea(mUp);	
		return EventBox::on_enter_notify_event(c);}
private:
	shared_ptr<MyAdjustment> mMyAdjustment;
	bool mUp;

	MyPageScrollButton * mMyPageScrollButton; // managed by gtk, hence no shared_ptr
	MySmoothScrollButton * mMySmoothScrollButton; // managed by gtk, hence no shared_ptr
	GoToTopOrBottomButton * mMyGoToTopOrBottomButton; // managed by gtk, hence no shared_ptr
};

void MyAdjustment::ScrollPage(bool aUp){
	mScrollingState = I_AM_NOT_SCROLLING;
	if (aUp){
		double lDesiredValue = get_value() - get_page_size() + 27;
		if (lDesiredValue >= get_lower())
			set_value(lDesiredValue);
		else{
			set_value(get_lower());
			mUpBox->YouHaveReachedTheScrollingLimitAndYouMustBecomeInsensitive(true, "scrollpage");
			/* when the mouse exits the button area, we will hide the box*/}}
	else{
		double lDesiredValue = get_value() + get_page_size() - 27;
		if (lDesiredValue <= get_upper() - get_page_size())
			set_value(lDesiredValue);
		else{
			set_value(get_upper() - get_page_size());
			mDownBox->YouHaveReachedTheScrollingLimitAndYouMustBecomeInsensitive(true, "scrollpage");
			/*when the mouse exits the button area, we will hide the box*/	}}}

void MyAdjustment::SetScrollButtonBoxes(MyScrollButtonBox * aUp, 
					MyScrollButtonBox* aDown) { 
	mUpBox = aUp;
	mDownBox = aDown;
	ShowOrHideTheScrollButtonsAfterHidingPolicyChanged();

}


void MyAdjustment::SetKeepScrollingBoxesAlwaysVisible(bool aKeep){
	if (aKeep){
		mCurrentScrollBoxesHidingPolicy = shared_ptr<ScrollBoxesHidingPolicy> (new ScrollBoxesHidingPolicyDoNotHide());}
	else{
		mCurrentScrollBoxesHidingPolicy = shared_ptr<ScrollBoxesHidingPolicy> (new ScrollBoxesHidingPolicyHide());}

	ShowOrHideTheScrollButtonsAfterHidingPolicyChanged();
}

void MyAdjustment::TheScrolledWindowHasBeenHidden(){
	mUpBox->hide();
	mDownBox->hide();}
void MyAdjustment::TheScrolledWindowHasBeenShown(){
	mUpBox->show();
	mDownBox->show();}

void MyAdjustment::GoToTopOrBottomButtonClicked(bool aTop){
	ScrollBoxesHidingPolicyHide * l = dynamic_cast<ScrollBoxesHidingPolicyHide*>(mCurrentScrollBoxesHidingPolicy.get());
	if (aTop){
		if (l!=0)
			mUpBox->hide();
		set_value(get_lower());
		//mUpBox->set_sensitive(false);

	}
	else{
		if (l!=0)
			mDownBox->hide();
		set_value(get_upper() - get_page_size());
		//mDownBox->hide();
		//mDownBox->set_sensitive(false);
	}
	mScrollingState = I_AM_NOT_SCROLLING;}

void MyAdjustment::TheMouseEnteredTheButtonArea(bool aUp){
	ScrollBoxesHidingPolicyHide * l = dynamic_cast<ScrollBoxesHidingPolicyHide*>(mCurrentScrollBoxesHidingPolicy.get());
	if (l != 0){ // if the current policy is to hide...
		if (aUp){
			l->mTheMouseIsNotOnTheTopButtonBox = false;}
		else{
			l->mTheMouseIsNotOnTheBottomButtonBox = false;}}}

void MyAdjustment::TheMouseLeftTheButtonArea(bool aUp){
	//cout << "TheMouseLeftTheButtonArea " <<endl;
	ScrollBoxesHidingPolicyHide * l = dynamic_cast<ScrollBoxesHidingPolicyHide*>(mCurrentScrollBoxesHidingPolicy.get());
	if (l != 0){ // if the current policy is to hide...
		if(aUp){
			l->mTheMouseIsNotOnTheTopButtonBox = true; 
			if(get_value() == get_lower()){
				mUpBox->hide(); // calls on_changed()
				set_value(get_lower());}}
		else{
			l->mTheMouseIsNotOnTheBottomButtonBox = true; 
			if(get_value()==get_upper()-get_page_size()){
				mDownBox->hide(); // calls on_changed()
				set_value(get_upper()-get_page_size());}}}}

// this is called at each frame during smooth scrolling
void MyAdjustment::on_value_changed(){
	//cout <<  "on_value_changed enters" <<endl;
	Adjustment::on_value_changed();

	ScrollBoxesHidingPolicyHide * l = dynamic_cast<ScrollBoxesHidingPolicyHide*>(mCurrentScrollBoxesHidingPolicy.get());
	
	if (get_value() < get_upper() - get_page_size()){
		if (!mDownBox->is_visible()){
			//cout << "showing downbox" <<endl;
			mDownBox->show(); /*this may call on_changed()!*/}
		mDownBox->YouHaveReachedTheScrollingLimitAndYouMustBecomeInsensitive(false, "on_value_changed");}
 	else{
		mDownBox->YouHaveReachedTheScrollingLimitAndYouMustBecomeInsensitive(true, "on_value_changed");
		if (l != 0){ // if the current policy is to hide...
			if (l->mTheMouseIsNotOnTheBottomButtonBox){
				//cout << "hiding downbox" <<endl;
				mDownBox->hide(); /* calls on_changed()*/}}}
	if (get_value() > get_lower() ){
		if (!mUpBox->is_visible()){
			//cout << "showing upbox" <<endl;
			mUpBox->show(); }
		mUpBox->YouHaveReachedTheScrollingLimitAndYouMustBecomeInsensitive(false, "on_value_changed");}
 	else{
		mUpBox->YouHaveReachedTheScrollingLimitAndYouMustBecomeInsensitive(true, "on_value_changed");
		if (l != 0){ // if the current policy is to hide...
			if (l->mTheMouseIsNotOnTheTopButtonBox){
				//cout << "hiding upbox" <<endl;
				mUpBox->hide();}}} // calls on_changed()
	//cout <<  "on_value_changed exits" <<endl;
}



void MyAdjustment::on_changed(){
	//cout <<  "on_changed enters" <<endl;
	Adjustment::on_changed();

	ScrollBoxesHidingPolicyHide * l = dynamic_cast<ScrollBoxesHidingPolicyHide*>(mCurrentScrollBoxesHidingPolicy.get());

	if (mDownBox){
		if (get_value() < get_upper() - get_page_size()){
			if (!mDownBox->is_visible())
				mDownBox->show();
			mDownBox->YouHaveReachedTheScrollingLimitAndYouMustBecomeInsensitive(false, "on_changed");}
 		else{
			mDownBox->YouHaveReachedTheScrollingLimitAndYouMustBecomeInsensitive(true, "on_changed");
			if (l != 0){ // if the current policy is to hide...
				if (l->mTheMouseIsNotOnTheBottomButtonBox)
					mDownBox->hide();}}
	}
	if (mUpBox){
		if (get_value() > get_lower()){
			if (!mUpBox->is_visible())
				mUpBox->show();
			mUpBox->YouHaveReachedTheScrollingLimitAndYouMustBecomeInsensitive(false, "on_changed");}
 		else{
			mUpBox->YouHaveReachedTheScrollingLimitAndYouMustBecomeInsensitive(true, "on_changed");
			if (l != 0){ // if the current policy is to hide...
				if (l->mTheMouseIsNotOnTheTopButtonBox)
					mUpBox->hide();}}}
	//cout <<  "on_changed exits" <<endl;
}

MyScrollButtonBox::MyScrollButtonBox(shared_ptr<MyAdjustment> aAdj, bool aUp){
	mMyAdjustment = aAdj;
	mUp=aUp;
	HBox * lHBox = manage(new HBox());
	lHBox->show();
	add(*lHBox);

	mMySmoothScrollButton = manage (new MySmoothScrollButton(aAdj, aUp));
	lHBox->pack_start(*mMySmoothScrollButton, false, true);

	mMyPageScrollButton = manage(new MyPageScrollButton(mMyAdjustment, aUp));
	lHBox->pack_start(*mMyPageScrollButton, true, true);

	mMyGoToTopOrBottomButton = manage(new GoToTopOrBottomButton(mMyAdjustment, aUp));
	lHBox->pack_start(*mMyGoToTopOrBottomButton, false, true);
}

void MyScrollButtonBox::YouHaveReachedTheScrollingLimitAndYouMustBecomeInsensitive(bool aReached, string callerName /*debug*/){
	//cout << "YouHaveReachedTheScrollingLimitAndYouMustBecomeInsensitive "<<(aReached?"true" : "false")<<" called by "<<callerName << endl;
	if (!aReached){
		if (!is_sensitive()){
			//cout << (mUp?"Top box" : "Bottom box") <<": setting button box sensitive"<<endl;
			set_sensitive(true);}
	}
	else{

		if (is_sensitive()){
			// REGION depress all buttons before set_sensitive(false), otherwise they get stuck pressed
			mMySmoothScrollButton->released();
			mMyPageScrollButton->released();
			// END
			//cout << (mUp?"Top box" : "Bottom box") <<":setting button box insensitive"<<endl;
			set_sensitive(false);}
	}
	// REGION fixes gtk bug. Only needed if some of the button are hidden in the above code.
	// 	mMyPageScrollButton->queue_resize();
	// 	mMySmoothScrollButton->queue_resize();
	// 	mMyPageScrollButton->queue_resize();
	// END
}


void MyAdjustment::ShowOrHideTheScrollButtonsAfterHidingPolicyChanged(){
	ScrollBoxesHidingPolicyHide * l = dynamic_cast<ScrollBoxesHidingPolicyHide*>(mCurrentScrollBoxesHidingPolicy.get());
	if (l==0){
		mUpBox->show();
		mDownBox->show();
	}
	else{
		on_value_changed();
	}
}
MyAdjustment::MyAdjustment(string name) : Adjustment( 0, 0, 1){ 
	mName = name;
	mScrollingState = I_AM_NOT_SCROLLING;
	//Glib::signal_idle().connect( SigC::slot(*this, &MyAdjustment::on_idle) , PRIORITY_LOW); // cpu bloat
	Glib::signal_timeout().connect( SigC::slot(*this, &MyAdjustment::on_idle) , 20);
	mUpBox = 0;
	mDownBox  = 0;
	mCurrentScrollBoxesHidingPolicy = shared_ptr<ScrollBoxesHidingPolicy>(new ScrollBoxesHidingPolicyHide());
}


void MyAdjustment::Update(){
	//cout << "the request was fillfilled, doing another"<<endl;
	if (mScrollingState == I_AM_SCROLLING_DOWN){
		double lNextValue;
		//ScrollBoxesHidingPolicyHide * l = dynamic_cast<ScrollBoxesHidingPolicyHide*>(mCurrentScrollBoxesHidingPolicy.get());
		//if (l!=0  && get_value()==0){
		// this fixes the scroll jump when you smooth scroll down and 
		// the top scroll box is hidden and suddenly appears.
		// the bug does NOT happen when scrolling up.
		//lNextValue  = gScrollButtonHeight /*scrollbuttonbox height @@ remove the hard coding*/ +
		//mScrollCurrentSpeed;}
		//			else{
		lNextValue = get_value()+ mScrollCurrentSpeed;
		//}
		if (lNextValue < get_upper() - get_page_size())
			set_value(lNextValue);
		else {
			set_value(get_upper() - get_page_size());
			mDownBox->YouHaveReachedTheScrollingLimitAndYouMustBecomeInsensitive(true, "update");
			// REGION necessary because the hidden button will not detect a on_button_release_event
			mScrollingState = I_AM_NOT_SCROLLING; 
			// END
		}
	}
	else if (mScrollingState == I_AM_SCROLLING_UP){
		double lNextValue = get_value()- mScrollCurrentSpeed;
		if (lNextValue >= get_lower())
			set_value(lNextValue);
		else /*if (lNextValue<get_lower())*/{
			set_value(get_lower() );
			mUpBox->YouHaveReachedTheScrollingLimitAndYouMustBecomeInsensitive(true, "update");
			// REGION necessary because the hidden button will not detect a on_button_release_event
			mScrollingState = I_AM_NOT_SCROLLING; 
			// END
		}
	}
}


// a custom char_traits that compares strings case-insensitively.
struct ignorecase_traits : public std::char_traits<char> {
    // return whether c1 and c2 are equal
    static bool eq(const char& c1, const char& c2) {
        return std::toupper(c1)==std::toupper(c2);
    }
    // return whether c1 is less than c2
    static bool lt(const char& c1, const char& c2) {
        return std::toupper(c1)<std::toupper(c2);
    }
    // compare up to n characters of s1 and s2
    static int compare(const char* s1, const char* s2,
                       std::size_t n) {
        for (std::size_t i=0; i<n; ++i) {
            if (!eq(s1[i],s2[i])) {
                return lt(s1[i],s2[i])?-1:1;
            }
        }
        return 0;
    }
    // search c in s
    static const char* find(const char* s, std::size_t n,
                            const char& c) {
        for (std::size_t i=0; i<n; ++i) {
            if (eq(s[i],c)) {
                return &(s[i]);
            }
        }
        return 0;
    }
};

// comparator that takes two boxes and decides which one must be
// displayed first, according to the name of the labels contained in
// the box.

// REGION given the HBox, it is too difficult to reach the filename
// within. I use this map.
std::map<GtkBox*, string> * gFromBoxToFullName;
// END

gint comparerByNameCaseInsensitive(gconstpointer a, gconstpointer b){
	GtkBoxChild * ch1 = (GtkBoxChild*)a;
	GtkBoxChild * ch2 = (GtkBoxChild*)b;

	GtkBox* b1 = GTK_BOX(ch1->widget);
	GtkBox* b2 = GTK_BOX(ch2->widget);

	// b1 and b2 are Item *
	if (gFromBoxToFullName ==0)
		cout << "error: gFromBoxToFullName = null"<<endl;

        std::map<GtkBox*, string> & lFromBoxToFullName = *gFromBoxToFullName;
	const string id1 = lFromBoxToFullName[b1];
	const string id2 = lFromBoxToFullName[b2];

	// REGION extract the basenames and put them into
	// case-insensitive-strings. I cannot simply use stl::string,
	// because then the compare function would compare them in a
	// case-sensitive way.
	basic_string<char, ignorecase_traits> lBasename1 = basename(id1.c_str());
	basic_string<char, ignorecase_traits> lBasename2 = basename(id2.c_str());
	// END

	return lBasename1.compare(lBasename2); // compares case-insensitive.

}

gint comparerByDate(gconstpointer a, gconstpointer b){
	GtkBoxChild* ch1 =  (GtkBoxChild*)a;
	GtkBoxChild* ch2 =(GtkBoxChild*)b;

	GtkBox* b1 = GTK_BOX(ch1->widget);
	GtkBox* b2 = GTK_BOX(ch2->widget);
	// b1 and b2 are Item *

	if (gFromBoxToFullName ==0)
		cout << "error: gFromBoxToFullName = null"<<endl;
	std::map<GtkBox*, string>& lFromBoxToFullName = *gFromBoxToFullName;
	const string id1 = lFromBoxToFullName[b1];
	const string id2 = lFromBoxToFullName[b2];

	// REGION extract the  date 
	struct stat lStat1, lStat2;
	stat(id1.c_str(), &lStat1); 
	stat(id2.c_str(), &lStat2); 
	time_t lDate1 = lStat1.st_mtime;
	time_t lDate2 = lStat2.st_mtime;
	// END
	if (lDate1 > lDate2)
		return -1;
	else if (lDate1 < lDate2)
		return 1;
	else 
		return 0;

}

gint comparerBySize(gconstpointer a, gconstpointer b){
	GtkBoxChild* ch1 =  (GtkBoxChild*)a;
	GtkBoxChild* ch2 = (GtkBoxChild*)b;

	GtkBox* b1 = GTK_BOX(ch1->widget);
	GtkBox* b2 =  GTK_BOX(ch2->widget);

	if (gFromBoxToFullName ==0)
		cout << "error: gFromBoxToFullName = null"<<endl;
	std::map<GtkBox*, string>& lFromBoxToFullName = *gFromBoxToFullName;

	const string id1 = lFromBoxToFullName[b1];
	const string id2 = lFromBoxToFullName[b2];

	// REGION extract the size
	struct stat lStat1, lStat2;
	stat(id1.c_str(), &lStat1); 
	stat(id2.c_str(), &lStat2); 
	off_t lSize1 = lStat1.st_size;
	off_t lSize2 = lStat2.st_size;
	// END
	if (lSize1 > lSize2)
		return -1;
	else if (lSize1 < lSize2)
		return 1;
	else 
		return 0;

}


// a SegWidget is a Gtk::Widget which can be created/referred-to in
// prolog. It is a widget which has a prolog name. It has innate
// capability to add itself to the global list of widgets and to
// remove itself from the list when he is destroyed.
//
// this class is a policy (or strategy). It is meant to be inherited
// (implementation inheritance).
template <class Widg/*a Gtk::Widget*/> class SegWidget : public Widg{
public:
	//  All widgets, when destroyed, (e.g. because their parent
	// was destroyed) must remove themselves from the global
	// map. This is why we cannot use shared_ptr.
	virtual ~SegWidget(){
		//cout << "SegWidget destroyed and unregistered: "<< mPrologName->ToString() <<endl; 
		gPrologNameToGtkWidget.erase(*mPrologName);}
	const shared_ptr<SegTerm> PrologName() const{
		return mPrologName;}
	bool WidthIsDefined(){ return mCurrentWidthInPixels !=0;}
	/// throws if WidthIsDefined()==false.
	int Width() { return *mCurrentWidthInPixels;}
protected:
	SegWidget(	/// all widgets have a prolog name, which is a term.
			const shared_ptr<SegTerm> aPrologName);
	shared_ptr<SegTerm> mPrologName;

	virtual void on_size_allocate(GtkAllocation* a){
		if (mCurrentWidthInPixels ==0){
			mCurrentWidthInPixels = shared_ptr<int>(new int);
		}
		*mCurrentWidthInPixels = a->width;
		Widg::on_size_allocate(a);
	}
private:
	shared_ptr<int> mCurrentWidthInPixels;
};

template<class Widg> SegWidget<Widg>::SegWidget(	/// all widgets have a prolog name, which is a term.
		const shared_ptr<SegTerm> aPrologName){
	//cout << "SegWidget::SegWidget " << aPrologName->ToString() << endl;
	mPrologName = aPrologName;
	// REGION add the widget to the map
	Widget * lPreviousWidgetWithSameName = gPrologNameToGtkWidget[*aPrologName];
	if (lPreviousWidgetWithSameName != 0){
		cout << ">>>>>> ERROR! SegWidget::SegWidget: a widget with the name " << aPrologName->ToString() << " already exists" << endl;
	}
	gPrologNameToGtkWidget[*aPrologName] = this;
	//cout << "Registered widget " << aPrologName->ToString() << endl;
	// END
	//cout << "SegWidget::SegWidget end" << endl;
}

class SegHandleBox : public SegWidget<HandleBox>{
public:
	SegHandleBox(const shared_ptr<SegTerm> aPrologName) : SegWidget<HandleBox>(aPrologName){}};

class SegScrolledWindow : public SegWidget<ScrolledWindow>{
public:
	SegScrolledWindow(const shared_ptr<SegTerm> aPrologName) : SegWidget<ScrolledWindow>(aPrologName){}};



class SegOptionMenu : public SegWidget<OptionMenu>{
public:
	SegOptionMenu(const shared_ptr<SegTerm> aPrologName) : SegWidget<OptionMenu>(aPrologName){
		mMenu  = shared_ptr<Menu> (new Menu()); 
		set_menu(*mMenu);	}
	void SetSelectedItem(const shared_ptr< SegTerm> aItemPrologNameAndData){
		set_history(mFromSegTermToInteger[*aItemPrologNameAndData]);	}
	void AddItem(const shared_ptr< SegTerm> aItemPrologNameAndData, 
		     const string& aItemText){

		mFromSegTermToInteger[*aItemPrologNameAndData] = mFromIntToSegTerm.size();
		mFromIntToSegTerm.push_back(*aItemPrologNameAndData);
			
		mMenu->items().push_back(Gtk::Menu_Helpers::MenuElem(
							aItemText)); //nothing must happen when the item is selected
	}

protected:
	virtual void on_changed(){
		int lNumber = get_history();

		term_t lTermToReturn = mFromIntToSegTerm[lNumber].ToPrologTerm();

		// REGION we must unify
		// gLatestEvent=menuItemClicked(mPrologName)
		functor_t lFunctor= PL_new_functor(PL_new_atom("optionMenuItemSelected"), 2);
		term_t lName  = this->mPrologName->ToPrologTerm();
		term_t gLatestEvent = PL_new_term_ref();
		PL_cons_functor(gLatestEvent, lFunctor, lName, lTermToReturn);
		gLatestEvents.push_back(SegTerm(gLatestEvent));
		// END
	}



private:
	shared_ptr<Menu> mMenu;
	std::map<SegTerm, int> mFromSegTermToInteger;
	vector<SegTerm> mFromIntToSegTerm;
};


class SegFrame : public SegWidget<Frame>{
public:
	SegFrame(const shared_ptr<SegTerm> aPrologName) : SegWidget<Frame>(aPrologName){}
};
class SegHSeparator : public SegWidget<HSeparator>{
public:
	SegHSeparator(const shared_ptr<SegTerm> aPrologName) : SegWidget<HSeparator>(aPrologName){}
};
class SegVSeparator : public SegWidget<VSeparator>{
public:
	SegVSeparator(const shared_ptr<SegTerm> aPrologName) : SegWidget<VSeparator>(aPrologName){}
};
class SegEntry : public SegWidget<Entry>{
public:
	SegEntry(const shared_ptr<SegTerm> aPrologName) : SegWidget<Entry>(aPrologName){}
protected:
	virtual void on_changed();
};

void SegEntry::on_changed(){
	// REGION we must unify gLatestEvent=entryTextChanged(entryName)
	functor_t lFunctor = PL_new_functor(PL_new_atom("entryTextChanged"), 1);
	term_t lEntryName  = this->mPrologName->ToPrologTerm();
	term_t gLatestEvent = PL_new_term_ref();
	PL_cons_functor(gLatestEvent, lFunctor, lEntryName);
	gLatestEvents.push_back(SegTerm(gLatestEvent));
	// END
}

class SegWindow : public SegWidget<Window>{
public:
	SegWindow(const shared_ptr<SegTerm> aPrologName) : SegWidget<Window>(aPrologName){
	
	}
protected:
	virtual bool on_delete_event(GdkEventAny * aEvent){

		// REGION send prolog message windowDestroyed(mPrologName)
		functor_t lFun = PL_new_functor(PL_new_atom("windowDestroyed"), 1);
		term_t gLatestEvent = PL_new_term_ref();
		PL_cons_functor(gLatestEvent, lFun, mPrologName->ToPrologTerm()); 
		gLatestEvents.push_back(SegTerm(gLatestEvent));
		// END
		delete this;
		return false; // accept deletion
	}


};

class SegEventLabel: public SegWidget<EventBox>{
public:
	SegEventLabel(const shared_ptr<SegTerm> aPrologName, string aText, bool aLineWrap) 
		: SegWidget<EventBox>(aPrologName)
	{
		mLabel = manage(new Label(aText));
		mLabel->show();
		mLabel->set_padding(12,-1); // @@ hardcoded

		add(*mLabel);
		if (aLineWrap)
			mLabel->set_line_wrap();

		//gTooltips->set_tip(*this,"test tip"); @@ doesn't work
	}
	void SetText(string t){
		mLabel->set_text(t);}
private:
	Label * mLabel; /// managed
};
class SegLabel : public SegWidget<Label>{
public:
	SegLabel(const shared_ptr<SegTerm> aPrologName, string aText) : SegWidget<Label>(aPrologName){
		set_text(aText);
	}
};
class SegImage : public SegWidget<Image>{
public:
	SegImage(const shared_ptr<SegTerm> aPrologName, string aFile) : SegWidget<Image>(aPrologName){
		set(aFile);
	}
};


// class SegToolbar : public SegWidget<Toolbar>{
// public:
// 	SegToolbar(const shared_ptr<SegTerm> aPrologName) : SegWidget<Toolbar>(aPrologName){
// 		set_toolbar_style(Gtk::TOOLBAR_BOTH);
// 		//set_toolbar_style(Gtk::TOOLBAR_BOTH_HORIZ);
// 		//set_toolbar_style(Gtk::TOOLBAR_ICONS);
// 		set_icon_size(IconSize(ICON_SIZE_SMALL_TOOLBAR));
// 	}

// 	void AddToggleButton(const shared_ptr<SegTerm> aItemPrologName,
// 		       string aImageFileName, 
// 		       string aText, 
// 		       string aTooltipText){
// 		mButtons.push_back(shared_ptr<SigC::Object>(new SegToolbarToggleButton(aItemPrologName,
// 										       aImageFileName,
// 										       aText,
// 										       aTooltipText,
// 										       *this)));
// 	}
// 	void AddButton(const shared_ptr<SegTerm> aItemPrologName,
// 		       string aImageFileName, 
// 		       string aText, 
// 		       string aTooltipText){
// 		mButtons.push_back(shared_ptr<SigC::Object>(new SegToolbarButton(aItemPrologName,
// 										aImageFileName,
// 										aText,
// 										aTooltipText,
// 										*this)));
// 	}
// 	void AddSpace(){
// 		tools().push_back(Gtk::Toolbar_Helpers::Space());
// 	}
// 	void HideButton(const shared_ptr<SegTerm> aItemPrologName){

// 	}
// private:
// 	std::list<shared_ptr<SigC::Object> >mButtons;

// 	class SegToolbarToggleButton : public SigC::Object{
// 	public:
// 		SegToolbarToggleButton(const shared_ptr<SegTerm> aPrologName,
// 				       string aImageFileName, 
// 				       string aText, 
// 				       string aTooltipText,
// 				       SegToolbar & aToolbar){
// 			mPrologName = aPrologName;
// 			mPressed = false;
// 			Image * lImage = manage(new Image(aImageFileName));
// 			lImage->show();
// 			aToolbar.tools().push_back(Gtk::Toolbar_Helpers::ToggleElem(
// 								   aText,
// 								   *lImage,
// 								   SigC::slot(*this, &SegToolbarToggleButton::onElementToggled),
// 								   aTooltipText));
// 		}
// 	private:
// 		shared_ptr<SegTerm> mPrologName;
// 		bool mPressed;
// 		void onElementToggled(){
// 			mPressed = !mPressed;
// 			// REGION we must unify
// 			// gLatestEvent=toolbarToggleButtonPressed(buttonName)
// 			// or toolbarToggleButtonReleased(buttonName)
// 			functor_t lFunctor;
// 			if (mPressed)
// 				lFunctor= PL_new_functor(PL_new_atom("toolbarToggleButtonPressed"), 1);
// 			else
// 				lFunctor= PL_new_functor(PL_new_atom("toolbarToggleButtonReleased"), 1);
// 			term_t lButtonName  = this->mPrologName->ToPrologTerm();
// 			term_t gLatestEvent = PL_new_term_ref();
// 			PL_cons_functor(gLatestEvent, lFunctor, lButtonName);
// 			gLatestEvents.push_back(SegTerm(gLatestEvent));
// 			// END
// 		}
// 	};
// 	class SegToolbarButton : public SigC::Object{
// 	public:
// 		SegToolbarButton(const shared_ptr<SegTerm> aPrologName,
// 				       string aImageFileName, 
// 				       string aText, 
// 				       string aTooltipText,
// 				       SegToolbar & aToolbar) {
// 			mPrologName = aPrologName;
// 			Image * lImage = manage(new Image(aImageFileName));
// 			lImage->show();
//  			aToolbar.tools().push_back(Gtk::Toolbar_Helpers::ButtonElem(
// 								   aText,
//  								   *lImage,
//  								   SigC::slot(*this, &SegToolbarButton::onElementPressed),
//  								   aTooltipText));
// 			//aToolbar.append_item(aText, aTooltipText, "shit", *lImage, SigC::slot(*this, &SegToolbarButton::onElementPressed));
// 		}
// 	private:
// 		shared_ptr<SegTerm> mPrologName;
// 		void onElementPressed(){
// 			// REGION we must unify
// 			// gLatestEvent=toolbarButtonPressed(buttonName)
// 			functor_t lFunctor= PL_new_functor(PL_new_atom("toolbarButtonPressed"), 1);
// 			term_t lButtonName  = this->mPrologName->ToPrologTerm();
// 			term_t gLatestEvent = PL_new_term_ref();
// 			PL_cons_functor(gLatestEvent, lFunctor, lButtonName);
// 			gLatestEvents.push_back(SegTerm(gLatestEvent));
// 			// END
// 		}
// 	};


// };

class SegVBox : public SegWidget<VBox>{
public:
	SegVBox(const shared_ptr<SegTerm> aPrologName) : SegWidget<VBox>(aPrologName){}
};
class SegHBox : public SegWidget<HBox>{
public:
	SegHBox(const shared_ptr<SegTerm> aPrologName) : SegWidget<HBox>(aPrologName){}
};
class SegButton : public SegWidget<Button>{
public:
	SegButton(const shared_ptr<SegTerm> aPrologName):SegWidget<Button>(aPrologName){
	}
protected:
	void on_clicked();
};

void SegButton::on_clicked(){
	cout << "button clicked: raising" <<endl;
	// REGION we must unify gLatestEvent=buttonClicked(buttonName)
	functor_t lFunctor = PL_new_functor(PL_new_atom("buttonClicked"), 1);
	term_t lButtonName  = this->mPrologName->ToPrologTerm();
	term_t gLatestEvent = PL_new_term_ref();
	PL_cons_functor(gLatestEvent, lFunctor, lButtonName);
	gLatestEvents.push_back(SegTerm(gLatestEvent));
	// END
}
class SegToggleButton : public SegWidget<ToggleButton>{
public:
	SegToggleButton(const shared_ptr<SegTerm> aPrologName,
			const string& aText):SegWidget<ToggleButton>(aPrologName){
		Label * lLabel = manage(new Label(aText));
		lLabel->show();
		lLabel->set_padding(9,-1); // @@ hardcoded
		add(*lLabel);}
protected:
	void on_toggled();
};

void SegToggleButton::on_toggled(){
	cout << "button clicked: raising" <<endl;
	// REGION we must unify gLatestEvent=buttonClicked(buttonName)
	functor_t lFunctor = PL_new_functor(PL_new_atom("buttonClicked"), 1);
	term_t lButtonName  = this->mPrologName->ToPrologTerm();
	term_t gLatestEvent = PL_new_term_ref();
	PL_cons_functor(gLatestEvent, lFunctor, lButtonName);
	gLatestEvents.push_back(SegTerm(gLatestEvent));
	// END
}

class SegImageButton : public SegWidget<Button>{
public:
	SegImageButton(const shared_ptr<SegTerm> aPrologName, string aImageFile, Glib::ustring aText);
	virtual void set_label(string aText);
protected:
	virtual void on_clicked();
private:
	Label * mLabel;
};

SegImageButton::SegImageButton(
		const shared_ptr<SegTerm> aPrologName, 
		string aImageFile, 
		Glib::ustring aText) : SegWidget<Button>(aPrologName){
	HBox * lBox = manage(new HBox());
	lBox->show();
	add(*lBox);
		
	Image * lImage = manage(new Image(aImageFile));
	lImage->show();
	if (aText!="")
		lBox->pack_start(*lImage, false, true);
	else
		lBox->pack_start(*lImage, true, true);
	mLabel = manage(new Label());
	mLabel->show();
	if (aText!="")
		lBox->pack_start(*mLabel, true, true);
	else
		lBox->pack_start(*mLabel, false, true);

	mLabel->set_text(aText);
}
void SegImageButton::set_label(string aText){
		mLabel->set_text(aText);}

void SegImageButton::on_clicked(){
	// REGION we must unify gLatestEvent=buttonClicked(buttonName)
	functor_t lFunctor = PL_new_functor(PL_new_atom("buttonClicked"), 1);
	term_t lButtonName  = this->mPrologName->ToPrologTerm();
	term_t gLatestEvent = PL_new_term_ref();
	PL_cons_functor(gLatestEvent, lFunctor, lButtonName);
	gLatestEvents.push_back(SegTerm(gLatestEvent));
	// END
}
class SegTextView : public SegWidget<ScrolledWindow>{
public:
	SegTextView(const shared_ptr<SegTerm> aPrologName) : SegWidget<ScrolledWindow>(aPrologName){
		set_policy(POLICY_AUTOMATIC, POLICY_ALWAYS);
		//set_shadow_type(SHADOW_NONE);
		mTextView = manage(new TextView());
		mTextView->show();
		add(*mTextView);


		mTextBuffer =  TextBuffer::create();

		mTextBuffer->set_text("This is the log. It is still empty. It will be filled when you perform actions.");

		mTextView->set_buffer(mTextBuffer);
		set_size_request(-1, 125);
		mTextView->set_editable(false);
	}
	void AddLine(const string & s){
 		//cout << "SegTextView::Addline"<<endl;
 		mStringList.push_back(s);
 		// REGION create linearization of the string list
 		string lAppendedStrings="This is the log \n";
 		for(std::list<string>::const_iterator i = mStringList.begin();
 		    i!=mStringList.end(); i++){
			//string s2 = *i;
			//cout << "appending" <<s2 <<endl;
			lAppendedStrings+= *i;
 			lAppendedStrings+="\n\n";
 		}
 		// END
 		//cout << "SegTextView::Addline setting text..."<<lAppendedStrings<<endl;
 		mTextBuffer->set_text(lAppendedStrings);
 		//cout << "SegTextView::Addline ended"<<endl;

		// REGION scroll to end
		TextIter lIter = mTextBuffer->end();
		mTextView->scroll_to_iter(lIter, 0.0);
		// END
	}
private:
	Glib::RefPtr<TextBuffer> mTextBuffer;
	std::list<string> mStringList;
	TextView * mTextView; // managed
};

class SegImageToggleButton : public SegWidget<ToggleButton>{
public:
	SegImageToggleButton(const shared_ptr<SegTerm> aPrologName, 
			     string aImageFile, 
			     Glib::ustring aText):SegWidget<ToggleButton>(aPrologName){
		HBox * lBox = manage(new HBox());
		lBox->show();
		add(*lBox);
		
		Image * lImage = manage(new Image(aImageFile));
		lImage->show();
		if (aText!="")
			lBox->pack_start(*lImage, false, true);
		else
			lBox->pack_start(*lImage, true, true);
		
		mLabel = manage(new Label());
		mLabel->show();
		if (aText!="")
			lBox->pack_start(*mLabel, true, true);
		else
			lBox->pack_start(*mLabel, false, true);

		mLabel->set_text(aText);
	}
	virtual void set_label(string aText){
		mLabel->set_text(aText);}
protected:
	// callback
	virtual void on_toggled();
private:
	Label * mLabel;
};

void SegImageToggleButton::on_toggled(){
	// REGION we must unify gLatestEvent=buttonClicked(buttonName)
	functor_t lFunctor = PL_new_functor(PL_new_atom("buttonClicked"), 1);
	term_t lButtonName  = this->mPrologName->ToPrologTerm();
	term_t gLatestEvent = PL_new_term_ref();
	PL_cons_functor(gLatestEvent, lFunctor, lButtonName);
	gLatestEvents.push_back(SegTerm(gLatestEvent));
	// END
}

class VBoxWithSize : public VBox{
public:
	bool HeightIsDefined() const{
		return mCurrentHeightInPixels != 0;}
	int Height() const{
		return *mCurrentHeightInPixels;	}

protected:
	// In Gtk this is the only way to get the actual widget size! :-(
	virtual void on_size_allocate(GtkAllocation * a){
		if (mCurrentHeightInPixels ==0){
			mCurrentHeightInPixels = shared_ptr<int>(new int);
		}
		*mCurrentHeightInPixels = a->height;
		VBox::on_size_allocate(a);
	}
	
private:
	shared_ptr<int> mCurrentHeightInPixels;
};


class SegEventBox : public SegWidget<EventBox>{
public:
	SegEventBox(const shared_ptr<SegTerm> aPrologName) 
		: SegWidget<EventBox>(aPrologName){}
};

class SegCheckListBox : public SegWidget<VBoxWithSize>{
public:
	void SortItemsByName();
	void SortItemsBySize();
	void SortItemsByDate();
	void DefeatBug(){
		mScrolledWindowSelected->queue_resize();
		mScrolledWindowDeselected->queue_resize();}
	void SetItemFolders(const shared_ptr< SegTerm> aItemPrologNameAndData, 
			    const string & aFolders);
	void SetFoldersVisibility(bool aVisible);
	SegCheckListBox(const shared_ptr<SegTerm> aPrologName, 
			bool aRelief,
			/// for example "times" or "verbs"
			const string & aReadableRepresentationOfWhatIAmHolding,
			bool aShowTheSelectedList);
	bool ItemExists(const SegTerm& aItemPrologNameAndData){
		return ( mFromItemPrologNameToItem[aItemPrologNameAndData] != 0);}
	void SetScrollPositionToTheTop(){
		mMyAdjustment->SetScrollPositionToTheTop();}
	/// returns the sum of the heigths of the visible items
	/// (selected and unselected). It can return 0 if the height
	/// cannot yet be computed correctly (due to a GTK bad design:
	/// the height can only be obtained with signals). in that
	/// case, you'll have to try later. You probably want to put
	/// this check in a loop.
	int OptimalHeight();
	/// Throws if !ItemExists() 
	void DeleteItem(const SegTerm& aItemPrologNameAndData);
	/// Creates a new SegCheckListBox::Item
	void AddItem(const shared_ptr< SegTerm> aItemPrologNameAndData, 
		     term_t aItemText,
		     const Glib::RefPtr<Pango::Context> aContext,
		     const string & aTooltip,
		     const string & aFolders,
		     const string & aDir= "",
		     const string & aSize = "",
		     const string & aDate = "",
		     const string & aIconFileName= "");
	/// used to group items.
	void AddSeparator();
	enum ItemState{
		ITEM_STATE_SELECTED,
		ITEM_STATE_DESELECTED, 
		ITEM_STATE_HIDDEN
	};
	// throws if !ItemExists()
	void SetItemState(const shared_ptr<SegTerm> aItemPrologNameAndData, ItemState aItemState);
	// this must be called when the size of a selected item
	// changes (e.g. if directories are shown/hidden), when an
	// item is selected or deselected.
	void ResizeSelectedItemsList();

	void SetDateVisibility(bool aVisible);
	void SetDirVisibility(bool aVisible);
	void SetSizeVisibility(bool aVisible);
	bool TheDateIsVisible(){
		return mTheDateIsVisible;}
	bool TheDirIsVisible(){
		return mTheDirIsVisible;}
	bool TheSizeIsVisible(){
		return mTheSizeIsVisible;}
	/// sometimes it is useful to hide all the items and then show
	/// only a few.
	void HideAllItems();
	// deletes all the items.
	void Clear();

	void SetKeepScrollingBoxesAlwaysVisible(bool aKeep);
private:
	bool mTheDirIsVisible;
	bool mTheSizeIsVisible;
	bool mTheDateIsVisible;

	class Item;
	struct ItemPair;
	/// this is the box that containes the items. This is a ptr
	/// but it must not be deleted in the destructor nor wrapped
	/// in a shared_ptr. It will become a _managed_ child widget
	/// of the SegCheckListBox, so it will be automatically
	/// deleted.
	VBox * mVBoxUnselected;
	// contains the selected items
	VBox * mVBoxSelected;

	shared_ptr<MyAdjustment> mMyAdjustment;
	/// association between item name and item (actually there are
	/// two items, the selected one and the deselected one).
	std::map<SegTerm, shared_ptr<ItemPair> > mFromItemPrologNameToItem;
	/// needed for sorting. :-( It uses Gtk, not Gtkmm.
	std::map<GtkBox*, string> mFromBoxToFullName;

	// contains mScrolledWindowSelected and lSpacer
	VBox * mSelectedVBox;
	// this reference must be kept because we need to resize it
	ScrolledWindow * mScrolledWindowSelected;
	// this reference must be kept because we need to hide it
	ScrolledWindow * mScrolledWindowDeselected;
	
	/// used internally to compute the height of the selected
	/// items
	std::set<shared_ptr<Item> > mSelectedItems;
	/// whether the SegCheckListBox will have a shadow
	bool mRelief;


	// used this way: when mVisibleItemsCount ==
	// mSelectedItems.count(), then you can hide one of the
	// ScrolledWindows.
	int mVisibleItemsCount;

	//class ButtonForHidingSelectedItems;
	//ButtonForHidingSelectedItems* mHideSelectedItemsButton;
	//bool TheUserIsShowingTheSelectedItemsList() const;

	std::list<shared_ptr<HBox> > mSeparators;

	bool mTheFoldersAreVisible;
protected:
	// In Gtk this is the only way to get the actual widget size! :-(
	virtual void on_size_allocate(GtkAllocation * lAllocation);
};


void SegCheckListBox::AddSeparator(){
	HBox * lSeparator = new HBox(); // not managed;
	lSeparator->show();
	lSeparator->set_size_request(30,20);

	HSeparator * l = manage(new HSeparator());
	l->show();
	lSeparator->pack_start(*l, true, true, 25);
	mVBoxUnselected->pack_start(*lSeparator, false, false, 5);
	mSeparators.push_back(shared_ptr<HBox> (lSeparator));
}

class SegCheckListBox::Item :  public HBox{
public:
	/// An Item can have two kinds: above and below. THe item
	/// below is always visible, the one above is only visible
	/// when it is selected, and only sends deselected messages.
	class Kind;
	class KindAbove;
	class KindBelow;
	void SetFolders(const string & aFolders);
	Item(const Glib::RefPtr<Pango::Context> aContext,
	     const shared_ptr<Kind> aKind,
	     bool aIMustSetTheWhiteBackground,
	     /// needed for sorting? @@ maybe it's no more needed?
	     const shared_ptr<SegTerm> aPrologName, 
	     SegCheckListBox * aEnclosingCheckListBox, 
	     const Glib::ustring & aItemText, 
	     const string & aFolders,
	     bool aFoldersVisible,
	     const string & aDir,
	     const string & aSize,
	     const string & aDate,
	     const string & aIconFileName,
	     const string & aTooltip); 
	void SetFoldersVisibility(bool aVisible);
	void YouHaveBeenSelected();
	void YouHaveBeenDeselected();
	void SetWrapPixel(int p);
	void SetDirVisibility(bool aVisible);
	void SetSizeVisibility(bool aVisible);
	void SetDateVisibility(bool aVisible);
	const shared_ptr<SegTerm> PrologName() const{
		return mPrologName;}
	bool HeightIsDefined() const{
		return mCurrentHeightInPixels != 0;}
	int Height() const{
		return *mCurrentHeightInPixels;	}
private:
	class MyLabel ;
	class MyLabelSelected ;
	class MyLabelDeselected ;
	class MyEventBox ;

	// REGION these widgets are destroyed automatically when the
	// Item is destroyed, even though they are not shared_ptr,
	// because they are created with Gtk::manage().

	MyEventBox * mEventBox;
	// END

	/// A Item has a prolog name. If Item
	/// were a SegWidget this would be automatic, however
	/// Item cannot be a SegWidget, because it must
	/// not register itself in the global widget list.
	shared_ptr<SegTerm> mPrologName; 
	// I used shared_ptr because this int is not always valid.
	shared_ptr<int> mCurrentHeightInPixels;
	// we need a reference to the enclosing checklistbox in order
	// to send it some message when we are resized. I can't make
	// this a shared_ptr because I can't create the
	// SegCheckListBox with shared_ptr. It's not needed anyway,
	// because the destruction of the Item NEVER
	// triggers the destruction of the enclosing SegCheckListBox.
	SegCheckListBox* mCheckListBox;
protected:
	virtual void on_size_allocate(GtkAllocation* a) ;
}; // Item class

class SegCheckListBox::Item::Kind{
public:
	virtual ~Kind(){}
};
class SegCheckListBox::Item::KindAbove : public Kind{};
class SegCheckListBox::Item::KindBelow : public Kind{
public:
	/// An Item which is of kind Below can be selected or
	/// deselected.
	bool mSelected;
	KindBelow(){ mSelected = false;}
};



/// I reprogrammed a label because standard labels don't wrap
/// correctly
class SegCheckListBox::Item::MyLabel : public Box{
public:
	MyLabel(string aText, const Glib::RefPtr<Pango::Context> aContext) ;
	void SetWrapPixel(int p);
	virtual void YouHaveBeenDeselected()=0;
	virtual void YouHaveBeenSelected()=0;
protected:
	virtual bool on_expose_event(GdkEventExpose*)=0;
	bool mSetBoxSizeAtTheNextEventExpose;
	Glib::RefPtr<Pango::Layout> mLayout;
private:

	shared_ptr<int> mCurrentWrapPixel;

};


class SegCheckListBox::Item::MyLabelSelected : public MyLabel{
public:
	MyLabelSelected(string aText, const Glib::RefPtr<Pango::Context> aContext)  : MyLabel(aText, aContext){}
	virtual void YouHaveBeenSelected(){}
	virtual void YouHaveBeenDeselected(){}
protected:
	virtual bool on_expose_event(GdkEventExpose*);

};
class SegCheckListBox::Item::MyLabelDeselected : public MyLabel{
public:
	MyLabelDeselected(string aText, const Glib::RefPtr<Pango::Context> aContext)  : MyLabel(aText, aContext){
		mSelected = false;
	}
	virtual void YouHaveBeenSelected();
	virtual void YouHaveBeenDeselected();
protected:
	virtual bool on_expose_event(GdkEventExpose*);
private:
	bool mSelected;

};

bool SegCheckListBox::Item::MyLabelDeselected::on_expose_event(GdkEventExpose*){
	// REGION Set again the box size on expose!
	// DON'T REMOVE THIS! It seems useless since
	// the size has been already set in
	// SetWrapPixel, but is is needed for the
	// inexplicable bug where the go button does
	// not appear...
	if (mSetBoxSizeAtTheNextEventExpose){
		int w,h;
		mLayout->get_pixel_size(w, h);
		set_size_request(w,h);
		mSetBoxSizeAtTheNextEventExpose = false;
	}
	// END
	
	this->get_window()->draw_layout( 
			//get_style()->get_text_gc(get_state()),
			mSelected? get_style()->get_mid_gc(STATE_NORMAL) : 
			get_style()->get_text_gc(STATE_NORMAL),
			31/*icon width*/, 5/*border height*/, mLayout);

	// return true to stop any further event handlers being called
	// to draw this area
	return true;
}
bool SegCheckListBox::Item::MyLabelSelected::on_expose_event(GdkEventExpose*){
	// REGION Set again the box size on expose!
	// DON'T REMOVE THIS! It seems useless since
	// the size has been already set in
	// SetWrapPixel, but is is needed for the
	// inexplicable bug where the go button does
	// not appear...
	if (mSetBoxSizeAtTheNextEventExpose){
		int w,h;
		mLayout->get_pixel_size(w, h);
		set_size_request(w,h);
		mSetBoxSizeAtTheNextEventExpose = false;
	}
	// END
	
	this->get_window()->draw_layout( 
			get_style()->get_text_gc(get_state()),
			31/*icon width*/, 6/*border height*/, mLayout);

	// return true to stop any further event handlers being called
	// to draw this area
	return true;
}

void SegCheckListBox::Item::MyLabelDeselected::YouHaveBeenSelected(){
	if (!mSelected){
		mSelected = true;
		queue_resize();}}
void SegCheckListBox::Item::MyLabelDeselected::YouHaveBeenDeselected(){
	if (mSelected){
		mSelected = false;
		queue_resize();}}


class SegCheckListBox::Item::MyEventBox : public EventBox{
public:
	MyEventBox( shared_ptr<Kind> aKind,
		    bool aIMustSetTheWhiteBackground,
		    const Glib::RefPtr<Pango::Context> aContext,
		    const Glib::ustring & aItemText, 
		    SegCheckListBox* aEnclosingCheckListBox,
		    Item* aEnclosingItem,
		    const string & aFolders,
		    bool aFoldersVisible,
		    const string & aDir,
		    const string & aSize,
		    const string & aDate,
		    const string & aIconFileName,
		    const string & aTooltip);
	void SetFoldersVisibility(bool aVisible){
		mFoldersAreVisible = aVisible;
		if (!aVisible){
			mFoldersLabel->hide();}
		else{
			if(mFoldersLabel->get_text() != string("[]")){
				mFoldersLabel->show();}
			else{
				mFoldersLabel->hide();}}}
	void SetDirVisibility(bool aVisible);
	void SetSizeVisibility(bool aVisible);
	void SetDateVisibility(bool aVisible);
	void YouHaveBeenSelected();
	void YouHaveBeenDeselected();
	void SetFolders(const string & aFolders);
	void SetWrapPixel(int p);
	bool IAmAbove(){
		KindAbove * l = dynamic_cast<KindAbove*>(mKind.get());
		return l!=0;
	}
	bool IAmSelected(){
		KindBelow * l = dynamic_cast<KindBelow*>(mKind.get());
		if (l==0){
			// if i am above, I am always selected
			//cout << "Iamselected:: I am above, so I am selected" <<endl;
			return true;
		}
		else{
			//cout << "IAMSELECTED:: I am below" <<endl;
			return l->mSelected;
		}
		
	}
protected:
	virtual bool on_button_press_event(GdkEventButton* event);
	virtual bool on_enter_notify_event(GdkEventCrossing *);
	virtual bool on_leave_notify_event(GdkEventCrossing *);
private:
	bool mFoldersAreVisible;
	Label * mDirLabel;
	Label * mFoldersLabel;
	Label * mSizeLabel;
	Label * mDateLabel;
	MyLabel * mNameLabel;
	shared_ptr<Kind> mKind;
	// REGIOn don't make shared_ptr ! we don t want the checklistboxitem and segchecklistbox to be deleted.
	SegCheckListBox* mCheckListBox;
	Item* mCheckListBoxItem;
	// END
	shared_ptr<Image> mIcon;

};


void SegCheckListBox::Item::YouHaveBeenDeselected(){
	mEventBox->YouHaveBeenDeselected();
}
void SegCheckListBox::Item::YouHaveBeenSelected(){
	mEventBox->YouHaveBeenSelected();
}
void SegCheckListBox::Item::MyEventBox::YouHaveBeenDeselected(){
	if (IAmAbove())	{
		cout << "You cannot select or deselect a MyEventBox of KindAbove"<< endl;
		throw "err";
	}
	KindBelow * l = dynamic_cast<KindBelow*>(mKind.get());
	if (l->mSelected){
		l->mSelected = false;
	

		// REGION this is not actually a state, it is more of an aspect
		//set_state(STATE_NORMAL);
		mNameLabel->YouHaveBeenDeselected();
		if (mIcon != 0){
			mIcon->set_sensitive(true);}
		// END

		Gdk::Color lGreen("dark green");
		mFoldersLabel->modify_fg(STATE_NORMAL, lGreen);
		mFoldersLabel->modify_fg(STATE_PRELIGHT, lGreen);}}
void SegCheckListBox::Item::MyEventBox::YouHaveBeenSelected(){
	if (IAmAbove())	{
		cout << "You cannot select or deselect a MyEventBox of KindAbove"<< endl;
		throw "err";
	}
	KindBelow * l = dynamic_cast<KindBelow*>(mKind.get());
	if (!l->mSelected){
		l->mSelected = true;
		if (mIcon != 0){
			mIcon->set_sensitive(false);}
		// REGION this is not actually a state, it is more of an aspect
		// I don't want below items to appear selected...
		//set_state(STATE_SELECTED);
		mNameLabel->YouHaveBeenSelected();
		// END
		mFoldersLabel->modify_fg(STATE_NORMAL, get_style()->get_mid(STATE_NORMAL));
		mFoldersLabel->modify_fg(STATE_PRELIGHT, get_style()->get_mid(STATE_NORMAL));}}

void SegCheckListBox::Item::MyLabel::SetWrapPixel(int p){
	// this is not the bottleneck: I tried commenting here and
	// speed is unchanged. The slowdown when you select an item at
	// the beginning of the list does not depend on this.


	// optimization
	if (mCurrentWrapPixel==0 || *mCurrentWrapPixel !=p){
		mLayout->set_width(p*Pango::SCALE);
		// REGION set box size NOW. If you
		// only do it in the expose event,
		// there is a glitch the first time
		// the label appears.
		int w,h;
		mLayout->get_pixel_size(w, h);
		set_size_request(w,h);
		// END 

		// REGION the box size will be set
		// again in on_expose_event
		mSetBoxSizeAtTheNextEventExpose = true; 
		// END
				
		// REGION remember the current wrap
		mCurrentWrapPixel = shared_ptr<int>(new int);
		*mCurrentWrapPixel = p;
		// END
	}
}

SegCheckListBox::Item::MyLabel::MyLabel(string aText, const Glib::RefPtr<Pango::Context> aContext) {
	//  create_pango_layout(aText) created wrong
	//  font sometimes :-( . So I had to pass the
	//  context.
	mLayout = Pango::Layout::create(aContext);
	mLayout->set_text(aText);
	mLayout->set_wrap(Pango::WRAP_WORD_CHAR);
	SetWrapPixel(100); 
	queue_resize();
}

void SegCheckListBox::Item::MyEventBox::SetDirVisibility(bool aVisible){
	if (aVisible)
		mDirLabel->show();
	else
		mDirLabel->hide();}
void  SegCheckListBox::Item::MyEventBox::SetSizeVisibility(bool aVisible){
			if (aVisible)
				mSizeLabel->show();
			else
				mSizeLabel->hide();}
void  SegCheckListBox::Item::MyEventBox::SetDateVisibility(bool aVisible){
			if (aVisible)
				mDateLabel->show();
			else
				mDateLabel->hide();}

void  SegCheckListBox::Item::MyEventBox::SetWrapPixel(int p){
	mNameLabel->SetWrapPixel(p);
	mDirLabel->set_size_request(p, -1);
	mFoldersLabel->set_size_request(p, -1);

	mSizeLabel->set_size_request(p, -1);
	mDateLabel->set_size_request(p, -1);

}



void SegCheckListBox::Item::SetWrapPixel(int p){
	if (p<=0){
		cout << "Item:: p = " << p<<endl;
		throw p;}
	int wrap = max(2, p- 45/*icon size*/);
	mEventBox->SetWrapPixel(wrap);

}



void  SegCheckListBox::Item::SetDirVisibility(bool aVisible){
	mEventBox->SetDirVisibility(aVisible);
}
void  SegCheckListBox::Item::SetSizeVisibility(bool aVisible){
	mEventBox->SetSizeVisibility(aVisible);
}
void  SegCheckListBox::Item::SetDateVisibility(bool aVisible){
	mEventBox->SetDateVisibility(aVisible);
}


void SegCheckListBox::Item::SetFolders(const string & aFolders){
	mEventBox->SetFolders(aFolders);}

void SegCheckListBox::Item::MyEventBox::SetFolders(const string & aFolders){
	mFoldersLabel->set_text(aFolders);
	if (aFolders != string("[]"))
		mFoldersLabel->show();
	else
		mFoldersLabel->hide();}

SegCheckListBox::Item::MyEventBox::MyEventBox( 
		shared_ptr<Kind> aKind,
		bool aIMustSetTheWhiteBackground,
		const Glib::RefPtr<Pango::Context> aContext,
		const Glib::ustring & aItemText, 
		SegCheckListBox* aEnclosingCheckListBox,
		Item* aEnclosingItem,
		const string & aFolders,
		bool aFoldersVisible,
		const string & aDir,
		const string & aSize,
		const string & aDate,
		const string & aIconFileName,
		const string & aTooltip)
{
	mKind = aKind;
	mCheckListBox = aEnclosingCheckListBox;
	mCheckListBoxItem = aEnclosingItem;

	if (aTooltip.size()>0)
		gTooltips->set_tip(*this, aTooltip);


	if (aIMustSetTheWhiteBackground){
		RefPtr<Style> lStyle = get_style();
		lStyle->set_bg_pixmap(STATE_NORMAL, RefPtr<Gdk::Pixmap>());
		lStyle->set_bg(STATE_NORMAL, lStyle->get_base(STATE_NORMAL));
		set_style(lStyle);
	}

	// structure:
	//
	//   this (EventBox)
	//       hbox
	//           image
	//           vbox
	//             label (name)
	//             label( dir)
	//             label (size)
	//             label (date)

	HBox * lHBox = manage(new HBox()); 
	lHBox->show();
	this->add(*lHBox);
		
	if (aIconFileName != string("")){
		mIcon = shared_ptr<Image>(new Image(aIconFileName));
		mIcon->set_size_request(26,26); // @@ hardcoded is bad, but for now...
		mIcon->show();
		lHBox->pack_start(*mIcon, false, true, 1);}
	

	VBox * lVBox = manage(new VBox()); 
	lVBox->show();
	lHBox->pack_start(*lVBox, false, true, 3/*otherwise the letter is too attached to the left border or icon*/);

	Glib::ustring lNormalizedText = "";
	if (aItemText.validate())
		lNormalizedText = aItemText;
	// 		else
	//  			cout<< "Non critical error in UTF8 validation. Instead of the file \"" << 
	//  				aItemText <<"\", you will see a button with an empty label.\n "<<endl;		

	if (IAmSelected())
		mNameLabel = manage(new MyLabelSelected(lNormalizedText, aContext));
	else
		mNameLabel = manage(new MyLabelDeselected(lNormalizedText, aContext));
	mNameLabel->show();
	lVBox->pack_start(*mNameLabel, false, false, 4);


	mFoldersLabel = manage(new Label(aFolders));
	Gdk::Color mGreen("dark green");
	mFoldersLabel->modify_fg(STATE_NORMAL, mGreen);
	mFoldersLabel->set_line_wrap();
	mFoldersLabel->set_size_request(100, -1);
	lVBox->pack_start(*mFoldersLabel, false, false);

	if (aFoldersVisible && aFolders != string("[]"))
		mFoldersLabel->show();
      


	mDirLabel = manage(new Label(aDir));
	Gdk::Color mColor("blue");// = mDirLabel->get_style()->get_mid(STATE_NORMAL);
	mDirLabel->modify_fg(STATE_NORMAL, mColor);
	mDirLabel->set_line_wrap();
	mDirLabel->set_size_request(100, -1);
	lVBox->pack_start(*mDirLabel, false, false);
		
	mSizeLabel = manage(new Label(aSize));
	mSizeLabel->modify_fg(STATE_NORMAL, mColor);
	lVBox->pack_start(*mSizeLabel, false, false);
		
	mDateLabel = manage(new Label(aDate));
	mDateLabel->modify_fg(STATE_NORMAL, mColor);
	lVBox->pack_start(*mDateLabel, false, false);


}


SegCheckListBox::Item::Item(
		const Glib::RefPtr<Pango::Context> aContext,
		shared_ptr<Kind> aKind,
		bool aIMustSetTheWhiteBackground, 
		/// needed for sorting? @@
		const shared_ptr<SegTerm> aPrologName, 
		SegCheckListBox * aEnclosingCheckListBox, 
		const Glib::ustring & aItemText, 
		const string & aFolders,
		bool aFoldersVisible,
		const string & aDir,
		const string & aSize,
		const string & aDate,
		const string & aIconFileName,
		const string & aTooltip){
	mPrologName = aPrologName;
	mCheckListBox = aEnclosingCheckListBox;
	mEventBox = manage(new MyEventBox(
					   aKind, 
					   aIMustSetTheWhiteBackground,
					   aContext,
					   aItemText, 
					   aEnclosingCheckListBox, 
					   this,
					   aFolders,
					   aFoldersVisible,
					   aDir,
					   aSize, 
					   aDate, 
					   aIconFileName,
					   aTooltip));
	mEventBox->show();
	this->pack_start(*mEventBox, true, true);

}

void SegCheckListBox::Item::SetFoldersVisibility(bool aVisible){
	mEventBox->SetFoldersVisibility(aVisible);}

	
struct SegCheckListBox::ItemPair{
	ItemPair(const shared_ptr<Item> aSelected, const shared_ptr<Item> aDeselected){
		mSelected = aSelected;
		mDeselected = aDeselected;
	}
	shared_ptr<Item> mSelected;
	shared_ptr<Item> mDeselected;
};

void SegCheckListBox::SetFoldersVisibility(bool aVisible){
	mTheFoldersAreVisible = aVisible;
	for (std::map<SegTerm, shared_ptr<ItemPair> >::iterator i = mFromItemPrologNameToItem.begin();
	     i!= mFromItemPrologNameToItem.end(); i++){
		i->second->mSelected->SetFoldersVisibility(aVisible);
		i->second->mDeselected->SetFoldersVisibility(aVisible);}}


void SegCheckListBox::HideAllItems(){
	for (std::map<SegTerm, shared_ptr<ItemPair> >::iterator i = mFromItemPrologNameToItem.begin();
	     i!= mFromItemPrologNameToItem.end(); i++){
		i->second->mSelected->hide();
		i->second->mDeselected->hide();
	}
	mSelectedItems.clear(); //mNumberOfSelectedItems = 0;
	//cout << "mNumberOfSelectedItems = " << mNumberOfSelectedItems << endl;
	//mSelectedVBox->hide();
	mVisibleItemsCount = 0;
}

void SegCheckListBox::SetSizeVisibility(bool aVisible){
	for (std::map<SegTerm, shared_ptr<ItemPair> >::iterator i = mFromItemPrologNameToItem.begin();
	     i!= mFromItemPrologNameToItem.end(); i++){
		i->second->mSelected->SetSizeVisibility(aVisible);
		i->second->mDeselected->SetSizeVisibility(aVisible);

	}
	mTheSizeIsVisible = aVisible;
}

void SegCheckListBox::SetDirVisibility(bool aVisible){
	for (std::map<SegTerm, shared_ptr<ItemPair> >::iterator i = mFromItemPrologNameToItem.begin();
	     i!= mFromItemPrologNameToItem.end(); i++){
		i->second->mSelected->SetDirVisibility(aVisible);
		i->second->mDeselected->SetDirVisibility(aVisible);
	}
	mTheDirIsVisible = aVisible;
}

void SegCheckListBox::SetDateVisibility(bool aVisible){
	for (std::map<SegTerm, shared_ptr<ItemPair> >::iterator i = mFromItemPrologNameToItem.begin();
	     i!= mFromItemPrologNameToItem.end(); 
	     i++){
		i->second->mSelected->SetDateVisibility(aVisible);		
		i->second->mDeselected->SetDateVisibility(aVisible);		
	}
	mTheDateIsVisible = aVisible;
}


void SegCheckListBox::SetItemFolders(const shared_ptr< SegTerm> aItemPrologNameAndData, 
				     const string & aFolders){
	// REGION integrity check
	shared_ptr<ItemPair> l = mFromItemPrologNameToItem[*aItemPrologNameAndData];
	if (l==0)
		cout << "Error. The passed arg is not a Item: "<<aItemPrologNameAndData->ToString() <<endl;
	// END
	l->mSelected->SetFolders(aFolders);
	l->mDeselected->SetFolders(aFolders);}
void SegCheckListBox::AddItem(const shared_ptr< SegTerm> aItemPrologNameAndData, 
			      term_t aItemText,
			      const Glib::RefPtr<Pango::Context> aContext,
			      const string & aTooltip,
			      const string & aFolders,
			      const string & aDir,
			      const string & aSize,
			      const string & aDate,
			      const string & aIconFileName){
	mVisibleItemsCount++;
	// REGION integrity: make sure the item does not already exist
	shared_ptr<ItemPair> lAlreadyExisting = mFromItemPrologNameToItem[*aItemPrologNameAndData];
	if (lAlreadyExisting != 0){
		cout << "ERROR: the item already exists: "<<aItemPrologNameAndData->ToString()<<endl;
		throw string("error");
	}
	// END

	// REGION make sure the ustring is valid
	ustring aItemTextUTF = SegTerm::termToString(aItemText);
	
	if(!aItemTextUTF.validate()){
		char * lString;
		int lRet = PL_get_chars(aItemText, &lString, CVT_ALL | CVT_WRITE);
		if (lRet==FALSE){
			cout << "AddItem: error in PL_get_chars"<<endl;
			throw string();}
		else{
			cout << "ERROR: SegCheckListBox::AddItem: not a valid UTF string: " << lString << endl;
			throw string();}}
	// END

	shared_ptr<Item> lNewItemDeselected( // not managed.
			new Item(
					aContext,
					shared_ptr<SegCheckListBox::Item::Kind>(new SegCheckListBox::Item::KindBelow()),
					mRelief, // I must set the white background if relief
					aItemPrologNameAndData,
					this,
					aItemTextUTF, 
					aFolders,
					mTheFoldersAreVisible,
					aDir,
					aSize, 
					aDate, 
					aIconFileName,
					aTooltip));
	lNewItemDeselected->show();
	lNewItemDeselected->YouHaveBeenDeselected(); // bug fixed
	mVBoxUnselected->pack_start(*lNewItemDeselected.get(), false, true);

	// REGION fill the map needed by the sort function
	GtkBox* lBox = GTK_BOX(lNewItemDeselected->gobj());
	mFromBoxToFullName[lBox] = aItemPrologNameAndData->ToString();
	// END

	// REGION Since the deselected item is already visible, send
	// it a resize message! Fixes bug.
	lNewItemDeselected->SetWrapPixel(WidthIsDefined()?Width() : 100);
	// END

	// REGION add the same item in mVBoxUnselected, hidden
	shared_ptr<Item> lNewItemSelected (	
			new Item(
					aContext,
					shared_ptr<SegCheckListBox::Item::Kind>(new SegCheckListBox::Item::KindAbove()),
					false, // selected items never set the white background
					aItemPrologNameAndData,
					this,
					aItemTextUTF, 
					aFolders,
					mTheFoldersAreVisible,
					aDir,
					aSize, 
					aDate, 
					aIconFileName,
					aTooltip));
	mVBoxSelected->pack_start(*lNewItemSelected.get(), false, true);


	// REGION remember the association between the name and the selected/deselected items
	mFromItemPrologNameToItem[*aItemPrologNameAndData]  = 
		shared_ptr<ItemPair>(new ItemPair(lNewItemSelected, lNewItemDeselected));
	// END remember the association

	// REGION Send resize message to new item! Fixes bug.
	lNewItemSelected->SetWrapPixel(WidthIsDefined()?Width() : 100);
	// END

	// END

	lNewItemSelected->SetDirVisibility(mTheDirIsVisible);
	lNewItemDeselected->SetDirVisibility(mTheDirIsVisible);
	lNewItemSelected->SetDateVisibility(mTheDateIsVisible);
	lNewItemDeselected->SetDateVisibility(mTheDateIsVisible);
	lNewItemSelected->SetSizeVisibility(mTheSizeIsVisible);
	lNewItemDeselected->SetSizeVisibility(mTheSizeIsVisible);
}

void SegCheckListBox::DeleteItem(const SegTerm& aItemPrologNameAndData){
	cout << "SegCheckListBox::DeletItem: deleting item "<<aItemPrologNameAndData.ToString()<<endl;
	// REGION if the item was selected, remove it from the selected list
	// REGION find previous state of that item
	ItemState lPreviousState;
	if (mFromItemPrologNameToItem[aItemPrologNameAndData]->mSelected->is_visible())
		lPreviousState = ITEM_STATE_SELECTED;
	else if (mFromItemPrologNameToItem[aItemPrologNameAndData]->mDeselected->is_visible())
		lPreviousState = ITEM_STATE_DESELECTED;
	else
		lPreviousState = ITEM_STATE_HIDDEN;
	// END

	if (lPreviousState != ITEM_STATE_HIDDEN)
		mVisibleItemsCount--;

	if (lPreviousState==ITEM_STATE_SELECTED){
		mSelectedItems.erase(mFromItemPrologNameToItem[aItemPrologNameAndData]->mSelected) ; 
		cout << "mNumberOfSelectedItems = " << mSelectedItems.size() << endl;
 		if (mSelectedItems.size()==0)
 			mSelectedVBox->hide();
	}

	// END

	// REGION  erase from mFromBoxToFullName
	shared_ptr<ItemPair> lItem = mFromItemPrologNameToItem[aItemPrologNameAndData];
	if (lItem==0)
		cout<<  "ERROR! SegCheckListBox::DeleteItem: the passed value is not an item" << aItemPrologNameAndData.ToString() <<endl;
	GtkBox* lBox = GTK_BOX(lItem->mDeselected->gobj()); // we only sort deselected items :-)
	mFromBoxToFullName.erase(lBox);
	// END

	//REGION this destroys the shared_ptr<ItemPair>,
	//which destroys the ItemPair, which destroys the
	//two shared_ptr<Item>, which destroys the two
	//Item.
	mFromItemPrologNameToItem.erase(aItemPrologNameAndData); 
	// END
}

void SegCheckListBox::Clear(){


	// REGION this destroys all shared_ptr<ItemPair>,
	// which destroys all ItemPair, which destroys all
	// Item, which removes them from the VBox
	mFromItemPrologNameToItem.clear();
	// END

	// this removes the separators from mVBoxUnselected
	mSeparators.clear();

	mFromBoxToFullName.clear();
	mSelectedItems.clear(); 
	mSelectedVBox->hide();
	mVisibleItemsCount = 0;
}

void SegCheckListBox::SetKeepScrollingBoxesAlwaysVisible(bool aKeep){
         mMyAdjustment->SetKeepScrollingBoxesAlwaysVisible(aKeep);
}

void SegCheckListBox::on_size_allocate(GtkAllocation * lAllocation){
	cout << "on_size_allocate " << mPrologName->ToString() << endl;
	if (!WidthIsDefined() || Width() != lAllocation->width){
		for (std::map<SegTerm, shared_ptr<ItemPair> >::iterator i = mFromItemPrologNameToItem.begin();
		     i!= mFromItemPrologNameToItem.end(); i++){
			i->second->mSelected->SetWrapPixel( lAllocation->width);
			i->second->mDeselected->SetWrapPixel( lAllocation->width);
		}
	}
	SegWidget<VBoxWithSize>::on_size_allocate(lAllocation);
}

// class SegCheckListBox::ButtonForHidingSelectedItems : public Button{
// public:
// 	ButtonForHidingSelectedItems(ScrolledWindow * aVBoxThatIMustHideOrShow, 
// 				     const string & aReadableRepresentationOfWhatIAmHolding,
// 				     bool aSelectedListVisible){
// 		mVBoxThatIMustHideOrShow = aVBoxThatIMustHideOrShow;



// 		set_relief(RELIEF_NONE);
// 		mArrow = manage(new Arrow(ARROW_UP, SHADOW_IN));
// 		mArrow->show();
// 		add(*mArrow);
// 		mReadableRepresentationOfWhatIAmHolding = aReadableRepresentationOfWhatIAmHolding;

// 		if (aSelectedListVisible)
// 			ShowSelectedList();
// 		else
// 			HideSelectedList();
		
// 	}
// 	bool TheUserIsShowingTheSelectedItemsList() const{
// 		return mVBoxThatIMustHideOrShow->is_visible();
// 	}
// 	void on_clicked(){
// 		if (mVBoxThatIMustHideOrShow->is_visible()){
// 			HideSelectedList();
// 		}
// 		else{
// 			ShowSelectedList();
// 		}
			
// 	}

// private:
// 	void HideSelectedList(){
// 		mVBoxThatIMustHideOrShow->hide();
// 		mArrow->set(ARROW_DOWN, SHADOW_IN);
// 		gTooltips->set_tip(*this, "If you click here, a list will appear showing only the selected " + 
// 				   mReadableRepresentationOfWhatIAmHolding +
// 				   ". Useful if the list is very long and you want to quickly locate the selected " +
// 				   mReadableRepresentationOfWhatIAmHolding + ".");

// 	}
// 	void ShowSelectedList(){
// 		mVBoxThatIMustHideOrShow->show();
// 		mArrow->set(ARROW_UP, SHADOW_IN);
// 		gTooltips->set_tip(*this, "Hide the selected " + mReadableRepresentationOfWhatIAmHolding);
// 	}
// 	ScrolledWindow * mVBoxThatIMustHideOrShow; //just for talking
// 	string mReadableRepresentationOfWhatIAmHolding;
// 	Arrow * mArrow; // managed
// };


// bool SegCheckListBox::TheUserIsShowingTheSelectedItemsList() const{
// 	return mHideSelectedItemsButton->TheUserIsShowingTheSelectedItemsList();
// }

SegCheckListBox::SegCheckListBox(const shared_ptr<SegTerm> aPrologName, 
				 bool aRelief, 
				 /// for example "times" or "verbs"
				 const string & aReadableRepresentationOfWhatIAmHolding,
				 bool aShowTheSelectedList)
	: SegWidget<VBoxWithSize>(aPrologName){

	mTheFoldersAreVisible = true;
	mTheDirIsVisible = false;
	mTheSizeIsVisible = false;
	mTheDateIsVisible = false;
	// REGION create the scrolled windows

 	mSelectedVBox = manage(new VBox());


	mScrolledWindowSelected = manage(new ScrolledWindow());

	mScrolledWindowSelected->show();
	mSelectedVBox->pack_start(*mScrolledWindowSelected, false, true);
	mScrolledWindowSelected->set_policy(POLICY_NEVER, POLICY_AUTOMATIC);

	mScrolledWindowSelected->set_shadow_type(SHADOW_NONE);


	HBox * lSpacer = manage(new HBox());
	lSpacer->show();
	lSpacer->set_size_request(-1, 2);
	mSelectedVBox->pack_start(*lSpacer, false, true);


// 	mHideSelectedItemsButton = 
// 		manage(new ButtonForHidingSelectedItems(
// 				       mScrolledWindowSelected,
// 				       aReadableRepresentationOfWhatIAmHolding,
// 				       aShowTheSelectedList));
// 	mHideSelectedItemsButton->show();
// 	mSelectedVBox->pack_start(*mHideSelectedItemsButton, false, true);

	mMyAdjustment = shared_ptr<MyAdjustment> (new MyAdjustment(aPrologName->ToString() + "-deselected"));

	mScrolledWindowDeselected = manage(new MyScrolledWindow( //*new Adjustment(0,0,1),
							      mMyAdjustment));
	mScrolledWindowDeselected->show();

	mScrolledWindowDeselected->set_policy(POLICY_NEVER, POLICY_ALWAYS);
	mScrolledWindowDeselected->set_shadow_type(SHADOW_NONE);
	// END

	// REGION create the vertical box containing the selected items
	mVBoxSelected = manage(new VBox());
	mScrolledWindowSelected->add(*mVBoxSelected);
	mVBoxSelected->show();
	// END
	// REGION create the vertical box containing the unselected items
	mVBoxUnselected = manage(new VBox());
	mScrolledWindowDeselected->add(*mVBoxUnselected);
	mVBoxUnselected->show();
	// END

	// REGION scroll buttons
	MyScrollButtonBox * lUpButtonsBox = manage (new MyScrollButtonBox(mMyAdjustment, true/*up*/));
	MyScrollButtonBox * lDownButtonsBox = manage (new MyScrollButtonBox(mMyAdjustment, false/*bottom*/));
	mMyAdjustment->SetScrollButtonBoxes(lUpButtonsBox, lDownButtonsBox);
	// END

 	pack_start(*mSelectedVBox, false, true);
	pack_start(*lUpButtonsBox, false, true);
	pack_start(*mScrolledWindowDeselected, true, true);
	pack_start(*lDownButtonsBox, false, true);


	set_size_request(100,-1);
	// REGION set the shadow of the deselected window
	if (!aRelief)
		((Viewport*)mScrolledWindowDeselected->get_child())->set_shadow_type(SHADOW_NONE);
	// END
	// REGION set bg of deselected viewport
	if (aRelief){
		RefPtr<Style> lStyle = ((Viewport*)mScrolledWindowDeselected->get_child())->get_style();
		lStyle->set_bg_pixmap(STATE_NORMAL, RefPtr<Gdk::Pixmap>());
		lStyle->set_bg(STATE_NORMAL, lStyle->get_base(STATE_NORMAL));
		((Viewport*)mScrolledWindowDeselected->get_child())->set_style(lStyle);
	}
	// END
	// REGION set the shadow  of the selected window
	((Viewport*)mScrolledWindowSelected->get_child())->set_shadow_type(SHADOW_OUT);
	//((Viewport*)mScrolledWindowSelected->get_child())->set_state(STATE_SELECTED);
	// END


	mScrolledWindowDeselected->get_vscrollbar()->set_sensitive(false);
	mScrolledWindowDeselected->get_vscrollbar()->set_size_request(0, 0);
	//mScrolledWindowSelected->get_vscrollbar()->set_sensitive(false);
	//mScrolledWindowSelected->get_vscrollbar()->set_size_request(0, 0);
	mRelief = aRelief;
	mVisibleItemsCount = 0;


}

int SegCheckListBox::OptimalHeight(){
	int lTotalHeight = 0;
	for (std::set<shared_ptr<Item> >::const_iterator i = mSelectedItems.begin();
	     i!= mSelectedItems.end();     
	     i++){
		if ((*i)->HeightIsDefined())
			lTotalHeight+=(*i)->Height();	}
	for (std::map<SegTerm, shared_ptr<ItemPair> >::const_iterator i = mFromItemPrologNameToItem.begin();
	     i!=mFromItemPrologNameToItem.end();
	     i++){
		if ( i->second->mDeselected->is_visible()){
			if (i->second->mDeselected->HeightIsDefined())
				lTotalHeight+=i->second->mDeselected->Height(); 
			else{
				cout << "Height not defined for item " <<  i->second->mDeselected->PrologName()->ToString() << endl;
			}}}

	return lTotalHeight + 66 /*height of the top and bottom  ScrollButtons*/;
}
void SegCheckListBox::ResizeSelectedItemsList(){
	// REGION debug
	static int lCounter = 0;
	//cout << "calling ItemHeightChanged. Counter= " << lCounter++ <<endl;
	// END
	int lTotalHeightOfSelectedItems = 0;
	for (std::set<shared_ptr<Item> >::const_iterator i = mSelectedItems.begin();
	     i!= mSelectedItems.end();     i++){
		if ((*i)->HeightIsDefined())
			lTotalHeightOfSelectedItems+=(*i)->Height();	}

	int lTotalHeightOfSelectedItemsPlusFrame = lTotalHeightOfSelectedItems + 5; //@@
	int lChosenHeight;
	if (this->HeightIsDefined()){
		int lHalfWindowHeight = this->Height() /2;

		if (mSelectedItems.size() != mVisibleItemsCount)
			lChosenHeight= min(lTotalHeightOfSelectedItemsPlusFrame, lHalfWindowHeight);
		else
			lChosenHeight = min(lTotalHeightOfSelectedItemsPlusFrame, Height());
	}
	else 
		lChosenHeight = lTotalHeightOfSelectedItemsPlusFrame;
	mScrolledWindowSelected->set_size_request(-1, lChosenHeight);
}

void SegCheckListBox::SetItemState(const shared_ptr<SegTerm> aItemPrologNameAndData, ItemState aItemState) {
	// REGION integrity check
	shared_ptr<ItemPair> l = mFromItemPrologNameToItem[*aItemPrologNameAndData];
	if (l==0)
		cout << "Error. The passed arg is not a Item: "<<aItemPrologNameAndData->ToString() <<endl;
	// END
	// REGION find previous state of that item
	ItemState lPreviousState;
	if (l->mSelected->is_visible()){
		//cout << "previous state = selected"<< endl;
		lPreviousState = ITEM_STATE_SELECTED;
	}
	else if (l->mDeselected->is_visible()){
		//cout << "previous state = deselected"<< endl;
		lPreviousState = ITEM_STATE_DESELECTED;
	}
	else{
		lPreviousState = ITEM_STATE_HIDDEN;
	}
	// END
		
	if (aItemState == ITEM_STATE_SELECTED){
		if (!l->mSelected->is_visible()) // optimization
			l->mSelected->show();
		if (!l->mDeselected->is_visible()) // optimization
			l->mDeselected->show(); // Hiding is too slow! Sends on_size_allocate to all items BELOW it.
		l->mDeselected->YouHaveBeenSelected();
		// REGION show the selected listbox if there is at least one selected item
		if (lPreviousState != ITEM_STATE_SELECTED){
			mSelectedItems.insert(l->mSelected); 
			//mScrolledWindowSelected->show();
			mSelectedVBox->show();
			ResizeSelectedItemsList();
		}
		// END

		if (lPreviousState == ITEM_STATE_HIDDEN)
			mVisibleItemsCount++;
	}
	else if (aItemState == ITEM_STATE_DESELECTED){
		if (!l->mDeselected->is_visible()) // optimization
			l->mDeselected->show();
		l->mDeselected->YouHaveBeenDeselected();
		if (l->mSelected->is_visible()) // optimization
			l->mSelected->hide();
		// REGION hide the selected listbox if there is no item selected
		if (lPreviousState == ITEM_STATE_SELECTED){
			mSelectedItems.erase(l->mSelected);
			if (mSelectedItems.size()==0)
				//mScrolledWindowSelected->hide();
				mSelectedVBox->hide();
			ResizeSelectedItemsList();
		}
		// END

		if (lPreviousState == ITEM_STATE_HIDDEN)
			mVisibleItemsCount++;
	}
	else if (aItemState == ITEM_STATE_HIDDEN){
		if (l->mDeselected->is_visible()) // optimization
			l->mDeselected->hide();
		if (l->mSelected->is_visible()) // optimization
			l->mSelected->hide();
		// REGION hide the selected listbox if there is no item selected
		if (lPreviousState == ITEM_STATE_SELECTED){
			mSelectedItems.erase(l->mSelected);
			if (mSelectedItems.size()==0)
				//mScrolledWindowSelected->hide();
				mSelectedVBox->hide();
			ResizeSelectedItemsList();
		}
		// END

		if (lPreviousState != ITEM_STATE_HIDDEN)
			mVisibleItemsCount--;

	}
	else 
		throw string("unmanaged case");
//  	if (mVisibleItemsCount == mSelectedItems.size()){
//  		mScrolledWindowDeselected->hide(); 
// 		// very ugly: the scrolledwindow and the adjustment should be merged
// 		mMyAdjustment->TheScrolledWindowHasBeenHidden();
// 	}
//  	else{
//  		mScrolledWindowDeselected->show(); 
// 		// very ugly: the scrolledwindow and the adjustment should be merged
// 		mMyAdjustment->TheScrolledWindowHasBeenShown();}}
}

void SegCheckListBox::SortItemsByName(){
	// REGION find the GList with all Item
	gFromBoxToFullName = &mFromBoxToFullName; // read by comparerByNameCaseInsensitive
	mVBoxUnselected->children().glist() = g_list_sort(mVBoxUnselected->children().glist(), comparerByNameCaseInsensitive);
	// END
	mVBoxUnselected->queue_resize();
}
void SegCheckListBox::SortItemsBySize(){
	// REGION find the GList with all Item
	gFromBoxToFullName = &mFromBoxToFullName; // read by comparerBySize
	mVBoxUnselected->children().glist() = g_list_sort(mVBoxUnselected->children().glist(), comparerBySize);
	// END
	mVBoxUnselected->queue_resize();

}
void SegCheckListBox::SortItemsByDate(){
	// REGION find the GList with all Item
	gFromBoxToFullName = &mFromBoxToFullName; // read by comparerByDate
	mVBoxUnselected->children().glist() = g_list_sort(mVBoxUnselected->children().glist(), comparerByDate);
	// END
	mVBoxUnselected->queue_resize();	

}

void SegCheckListBox::Item::on_size_allocate(GtkAllocation* a) {
	// _Selected_ items must inform the enclosing SegCheckListBox
	// the list height must be changed
	if (mCurrentHeightInPixels ==0){
		mCurrentHeightInPixels = shared_ptr<int>(new int);
		*mCurrentHeightInPixels = a->height;
		if (mEventBox->IAmAbove()) //mIAmASelectedItem)
			mCheckListBox->ResizeSelectedItemsList();
	}
	else{
		if (*mCurrentHeightInPixels != a->height){
			*mCurrentHeightInPixels = a->height;
			if (mEventBox->IAmAbove())
				mCheckListBox->ResizeSelectedItemsList();
		}
	}
	HBox::on_size_allocate(a);
}

bool SegCheckListBox::Item::MyEventBox::on_enter_notify_event(GdkEventCrossing*){
	
	set_state(STATE_PRELIGHT);
	return true;
}
bool SegCheckListBox::Item::MyEventBox::on_leave_notify_event(GdkEventCrossing*){
	// 	if (IAmSelected())
 	//	set_state(STATE_SELECTED);
 	//else
 		set_state(STATE_NORMAL);

	
	return true;
}



bool SegCheckListBox::Item::MyEventBox::on_button_press_event(GdkEventButton*){
	// REGION unify gLatestEvent=checkListBoxItemSelected(checkListBoxName, buttonName)
	functor_t lFun;
	if (IAmAbove()){
		//cout << "I have been clicked while above" <<endl;
		lFun = PL_new_functor(PL_new_atom("checkListBoxItemDeselected"), 2);
	}
	else{
		//cout << "I have been clicked while below" <<endl;
		if (IAmSelected()){
			//cout << "I have been clicked while below and selected" <<endl;
			lFun = PL_new_functor(PL_new_atom("checkListBoxItemDeselected"), 2);
		}
		else{
			//cout << "I have been clicked while below and deselected" <<endl;
			lFun = PL_new_functor(PL_new_atom("checkListBoxItemSelected"), 2);
		}
	}
	// REGION bug fixed. without this, there is a strange bug with prolog.
	term_t gLatestEvent = PL_new_term_ref(); 
	// END
	PL_cons_functor(gLatestEvent, 
			lFun, 
			mCheckListBox->PrologName()->ToPrologTerm(), 
			mCheckListBoxItem->PrologName()->ToPrologTerm());
	gLatestEvents.push_back(SegTerm(gLatestEvent));
	// END

	// REGION tell the SegCheckListBox to select myself and my twin, or something like that..
	if (IAmAbove())
		// if i am above, then i am always selected, then I
		// have been deselected, therefore ask the
		// SegCheckListBox to hide me, deselect my twin, and
		// to update the selected window size:
		mCheckListBox->SetItemState(mCheckListBoxItem->PrologName(), SegCheckListBox::ITEM_STATE_DESELECTED);
	else{
		// if i am below, then I may have been selected or
		// deselected.
		if (IAmSelected())
			// this will change my mSelected value
			mCheckListBox->SetItemState(mCheckListBoxItem->PrologName(), SegCheckListBox::ITEM_STATE_DESELECTED);
		else
			// this will change my mSelected value
			mCheckListBox->SetItemState(mCheckListBoxItem->PrologName(), SegCheckListBox::ITEM_STATE_SELECTED);

	}
	// END
	return true;
}

class SegDialog: public SegWidget<Dialog>{
public:
	SegDialog(    const shared_ptr<SegTerm> aPrologName, 
		      const shared_ptr<SegTerm> aParent, 
		      string aTitle, 
		      string aIconFileName) :    SegWidget<Dialog>(aPrologName){
		set_title(aTitle);
		set_has_separator(false);
		set_modal(true);
		set_icon_from_file(aIconFileName);
	}  
	// never use widgetDestroy on the dialog! use only this.
	void Close(){
		DestroyMyselfAndPostPrologEvent();
	}
private:
	void DestroyMyselfAndPostPrologEvent(){
		functor_t lFun = PL_new_functor(PL_new_atom("dialogDestroyed"), 1);
		term_t gLatestEvent = PL_new_term_ref();
		PL_cons_functor(gLatestEvent, lFun, mPrologName->ToPrologTerm()); 
		gLatestEvents.push_back(SegTerm(gLatestEvent));
		// END
		delete this;
	}

protected:
	// REGION these events aren't raised when you call widgetDestroy on the dialog...

	// this is only called when the user closes the dialog via the
	// X button. Not when I call widgetDestroy(dialog) via code.
	virtual bool on_delete_event(GdkEventAny * aEvent){
		cout << "\n\n ON DELETE DIALOG CALLED. raising dialogDestroyed() \n\n";
		DestroyMyselfAndPostPrologEvent();		
		//return Dialog::on_delete_event(aEvent);
		return true;

	}
// 	virtual bool on_unmap_event(GdkEventAny * aEvent){
// 		cout << "\n\n ON unmap event DIALOG CALLED. raising dialogDestroyed() \n\n";
// 	}
// 	virtual void on_hide(){
// 		cout << "\n\n ON HIDE DIALOG CALLED.  \n\n";
// 	}
// 	virtual void on_unmap(){
// 		cout << "\n\n ON unmap DIALOG CALLED.  \n\n";
// 	}
// 	virtual void on_response(int id){
// 		cout << "\n\n ON response  CALLED. \n\n";
// 	}
// 	virtual void on_close(){
// 		cout << "\n\n ON close DIALOG CALLED.  \n\n";
// 	}
	// END


	/// dialog automatically center themselves when resized. @ it doesn't work :-(
// 	virtual void on_size_allocate(GtkAllocation* a){
// 		cout << "Dialog automatically centered" <<endl;
// 		Dialog::on_size_allocate(a);
// 		set_position(WIN_POS_CENTER);

// 	}
};



foreign_t handleBoxNew(term_t aRef) {
	
	SegHandleBox* l = new SegHandleBox(shared_ptr<SegTerm>(new SegTerm(aRef)));

	l->set_shadow_type(SHADOW_OUT);

	PL_succeed;
}
foreign_t frameNew(term_t aRef) {
	
	SegFrame* l = new SegFrame(shared_ptr<SegTerm>(new SegTerm(aRef)));

	l->set_shadow_type(SHADOW_OUT);

	PL_succeed;
}


foreign_t dialogNew(term_t aDialogRef, term_t aWindowRef, term_t aTitle, term_t aIcon) {
	SegDialog * lDialog= new SegDialog(shared_ptr<SegTerm>(new SegTerm(aDialogRef)),
					   shared_ptr<SegTerm>(new SegTerm(aWindowRef)),
					   SegTerm::termToString(aTitle),
					   SegTerm::termToString(aIcon));
	lDialog->set_position(WIN_POS_CENTER_ALWAYS);
	
	PL_succeed;
}

foreign_t checkListBoxClear(term_t aCheckListBoxPrologName){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aCheckListBoxPrologName)];
	if (lWidget==0){
		cout << "checkListBoxClear: The passed value is not a Widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	SegCheckListBox * lCheckListBox = dynamic_cast<SegCheckListBox*> (lWidget);
	// END
	if (lCheckListBox==0){
		cout << "The passed value is not a CheckListBox" << endl;
		PL_fail;}
	// END integrity checks
	lCheckListBox->Clear();
	PL_succeed;
}
foreign_t checkListBoxItemExists(term_t aCheckListBoxPrologName, term_t aItemPrologNameAndData){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aCheckListBoxPrologName)];
	if (lWidget==0){
		cout << "checkListBoxItemExists: the passed value is not a Widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	SegCheckListBox * lCheckListBox = dynamic_cast<SegCheckListBox*> (lWidget);
	// END
	if (lCheckListBox==0){
		cout << "The passed value is not a CheckListBox" << endl;
		PL_fail;}
	// END integrity checks
	if (lCheckListBox->ItemExists(SegTerm(aItemPrologNameAndData)))
		PL_succeed;
	else
		PL_fail;
}
foreign_t checkListBoxAddSeparator(term_t aCheckListBoxPrologName){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aCheckListBoxPrologName)];
	if (lWidget==0){
		cout << "checkListBoxSortByName: The passed value is not a Widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	SegCheckListBox * lCheckListBox = dynamic_cast<SegCheckListBox*> (lWidget);
	// END
	if (lCheckListBox==0){
		cout << "The passed value is not a CheckListBox" << endl;
		PL_fail;}
	// END integrity checks
	lCheckListBox->AddSeparator();
	PL_succeed;
}
foreign_t checkListBoxSortByName(term_t aCheckListBoxPrologName){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aCheckListBoxPrologName)];
	if (lWidget==0){
		cout << "checkListBoxSortByName: The passed value is not a Widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	SegCheckListBox * lCheckListBox = dynamic_cast<SegCheckListBox*> (lWidget);
	// END
	if (lCheckListBox==0){
		cout << "The passed value is not a CheckListBox" << endl;
		PL_fail;}
	// END integrity checks
	lCheckListBox->SortItemsByName();
	PL_succeed;
}
foreign_t checkListBoxSortBySize(term_t aCheckListBoxPrologName){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aCheckListBoxPrologName)];
	if (lWidget==0){
		cout << "checkListBoxSortBySize: The passed value is not a Widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	SegCheckListBox * lCheckListBox = dynamic_cast<SegCheckListBox*> (lWidget);
	// END
	if (lCheckListBox==0){
		cout << "The passed value is not a CheckListBox" << endl;
		PL_fail;}
	// END integrity checks
	lCheckListBox->SortItemsBySize();
	PL_succeed;
}
foreign_t checkListBoxSortByDate(term_t aCheckListBoxPrologName){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aCheckListBoxPrologName)];
	if (lWidget==0){
		cout << "checkListBoxSortByDate: The passed value is not a Widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	SegCheckListBox * lCheckListBox = dynamic_cast<SegCheckListBox*> (lWidget);
	// END
	if (lCheckListBox==0){
		cout << "The passed value is not a CheckListBox" << endl;
		PL_fail;}
	// END integrity checks
	lCheckListBox->SortItemsByDate();
	PL_succeed;
}


// foreign_t checkListBoxSetButtonCheckState(term_t aCheckListBoxPrologName, 
// 					  term_t aItemPrologName, 
// 					  term_t aChecked){
// 	// REGION integrity checks
// 	Widget* lWidget = 
// 		gPrologNameToGtkWidget[SegTerm(aCheckListBoxPrologName)];
// 	if (lWidget==0){
// 		cout << "checkListBoxSetButtonCheckState: The passed value is not a Widget" << endl;
// 		PL_fail;}
// 	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
// 	SegCheckListBox * lCheckListBox = dynamic_cast<SegCheckListBox*> (lWidget);
// 	// END
// 	if (lCheckListBox==0){
// 		cout << "The passed value is not a CheckListBox" << endl;
// 		PL_fail;}
// 	// END integrity checks
// 	bool lChecked = (SegTerm::termToString(aChecked) == string("checked"));
// 	lCheckListBox->SetItemCheckState(
// 			shared_ptr<SegTerm>(new SegTerm(aItemPrologName)), 
// 			lChecked);
// 	PL_succeed;
// }

foreign_t checkListBoxSetItemState(term_t aCheckListBoxPrologName, 
				    term_t aItemPrologName,
				    // 'selected' or 'deselected' or 'hidden'
				    term_t aState){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aCheckListBoxPrologName)];
	if (lWidget==0){
		cout << "checkListBoxSetItemVisibility: The passed value is not a Widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	SegCheckListBox * lCheckListBox = dynamic_cast<SegCheckListBox*> (lWidget);
	// END
	if (lCheckListBox==0){
		cout << "The passed value is not a CheckListBox" << endl;
		PL_fail;}
	// END integrity checks

	SegCheckListBox::ItemState lState = SegCheckListBox::ITEM_STATE_HIDDEN;
	if (SegTerm::termToString(aState) == string("selected"))
		lState = SegCheckListBox::ITEM_STATE_SELECTED;
	else if (SegTerm::termToString(aState) == string("deselected"))
		lState = SegCheckListBox::ITEM_STATE_DESELECTED;
	lCheckListBox->SetItemState(shared_ptr<SegTerm>(new SegTerm(aItemPrologName)), lState);
	PL_succeed;
 }

 foreign_t checkListBoxHideAllItems(term_t aCheckListBoxPrologName){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aCheckListBoxPrologName)];
	if (lWidget==0){
		cout << "checkListBoxHideAllItems: The passed value is not a Widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	SegCheckListBox * lCheckListBox = dynamic_cast<SegCheckListBox*> (lWidget);
	// END
	if (lCheckListBox==0){
		cout << "The passed value is not a CheckListBox" << endl;
		PL_fail;}
	// END integrity checks
	lCheckListBox->HideAllItems();
	PL_succeed;
 }
 foreign_t imageSet(term_t aImagePrologName, term_t aFileName){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aImagePrologName)];
	if (lWidget==0){
		cout << "imageSet: The passed value is not a Widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	Image * l = dynamic_cast<Image*> (lWidget);
	// END
	if (l==0){
		cout << "The passed value is not an Image" << endl;
		PL_fail;}
	// END integrity checks
	l->set(SegTerm::termToString(aFileName));
	PL_succeed;
 }

 foreign_t dialogAddToVBox(term_t aDialogPrologName, term_t aWidgetPrologName){
	// REGION integrity checks
	Widget* lWidgetDialog = 
		gPrologNameToGtkWidget[SegTerm(aDialogPrologName)];
	if (lWidgetDialog==0){
		cout << "dialogAddToVBox: The passed value is not a Widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	SegDialog * lDialog = dynamic_cast<SegDialog*> (lWidgetDialog);
	// END
	if (lDialog==0){
		cout << "The passed value is not a Dialog" << endl;
		PL_fail;}

	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aWidgetPrologName)];
	if (lWidget==0){
		cout << "The passed value is not a Widget" << endl;
		PL_fail;}
	// END integrity checks
	lDialog->get_vbox()->add(*lWidget);
	PL_succeed;
 }



// foreign_t toolbarAddToggleButton(term_t aToolbarBoxPrologName, 
// 				 term_t aButtonPrologName,
// 				 term_t aImageFile,
// 				 term_t aButtonText,
// 				 term_t aTooltipText){
// 	// REGION integrity checks
// 	Widget* lWidget = 
// 		gPrologNameToGtkWidget[SegTerm(aToolbarBoxPrologName)];
// 	if (lWidget==0){
// 		cout << "toolbarAddButton: The passed value is not a Widget" << endl;
// 		PL_fail;}
// 	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
// 	SegToolbar * l = dynamic_cast<SegToolbar*> (lWidget);
// 	// END
// 	if (l==0){
// 		cout << "The passed value is not a SegToolbar" << endl;
// 		PL_fail;}
// 	// END integrity checks
// 	l->AddToggleButton(shared_ptr<SegTerm> (new SegTerm(aButtonPrologName)), 
// 		     SegTerm::termToString(aImageFile), 
// 		     SegTerm::termToString(aButtonText),
// 		     SegTerm::termToString(aTooltipText));
// 	PL_succeed;
// }
// foreign_t toolbarAddButton(term_t aToolbarBoxPrologName, 
// 				 term_t aButtonPrologName,
// 				 term_t aImageFile,
// 				 term_t aButtonText,
// 				 term_t aTooltipText){
// 	// REGION integrity checks
// 	Widget* lWidget = 
// 		gPrologNameToGtkWidget[SegTerm(aToolbarBoxPrologName)];
// 	if (lWidget==0){
// 		cout << "toolbarAddButton: The passed value is not a Widget" << endl;
// 		PL_fail;}
// 	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
// 	SegToolbar * l = dynamic_cast<SegToolbar*> (lWidget);
// 	// END
// 	if (l==0){
// 		cout << "The passed value is not a SegToolbar" << endl;
// 		PL_fail;}
// 	// END integrity checks
// 	l->AddButton(shared_ptr<SegTerm> (new SegTerm(aButtonPrologName)), 
// 		     SegTerm::termToString(aImageFile), 
// 		     SegTerm::termToString(aButtonText),
// 		     SegTerm::termToString(aTooltipText));
// 	PL_succeed;
// }
// foreign_t toolbarAddSpace(term_t aToolbarBoxPrologName){
// 	// REGION integrity checks
// 	Widget* lWidget = 
// 		gPrologNameToGtkWidget[SegTerm(aToolbarBoxPrologName)];
// 	if (lWidget==0){
// 		cout << "toolbarAddButton: The passed value is not a Widget" << endl;
// 		PL_fail;}
// 	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
// 	SegToolbar * l = dynamic_cast<SegToolbar*> (lWidget);
// 	// END
// 	if (l==0){
// 		cout << "The passed value is not a SegToolbar" << endl;
// 		PL_fail;}
// 	// END integrity checks
// 	l->AddSpace();
// 	PL_succeed;
// }
// foreign_t toolbarAddLabel(term_t aToolbarBoxPrologName, term_t aText){
// 	// REGION integrity checks
// 	Widget* lWidget = 
// 		gPrologNameToGtkWidget[SegTerm(aToolbarBoxPrologName)];
// 	if (lWidget==0){
// 		cout << "toolbarAddButton: The passed value is not a Widget" << endl;
// 		PL_fail;}
// 	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
// 	SegToolbar * l = dynamic_cast<SegToolbar*> (lWidget);
// 	// END
// 	if (l==0){
// 		cout << "The passed value is not a SegToolbar" << endl;
// 		PL_fail;}
// 	// END integrity checks
// 	Label *lLabel = manage(new Label(SegTerm::termToString(aText)));
// 	lLabel->set_alignment(1.0f, 0.5f);
// 	lLabel->set_padding(12, 0);
// 	lLabel->show();
// 	l->tools().push_back(Gtk::Toolbar_Helpers::Element(*lLabel));

// 	PL_succeed;
// }
// set visibility of the size label, for ALL ITEMS in the CheckListBox.
foreign_t checkListBoxSetSizeVisibility(term_t aCheckListBoxPrologName, 
					// 'hidden' or 'visible'
					term_t aVisibility){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aCheckListBoxPrologName)];
	if (lWidget==0){
		cout << "checkListBoxSetSizeVisibility: The passed value is not a Widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	SegCheckListBox * lCheckListBox = dynamic_cast<SegCheckListBox*> (lWidget);
	// END
	if (lCheckListBox==0){
		cout << "The passed value is not a CheckListBox" << endl;
		PL_fail;}
	// END integrity checks
	bool lVisible = (SegTerm::termToString(aVisibility) == string("visible"));
	lCheckListBox->SetSizeVisibility(lVisible);
	PL_succeed;
}



foreign_t checkListBoxTheDateIsVisible(term_t aCheckListBoxPrologName){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aCheckListBoxPrologName)];
	if (lWidget==0){
		cout << "checkListBoxSetDateVisibility: The passed value is not a Widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	SegCheckListBox * lCheckListBox = dynamic_cast<SegCheckListBox*> (lWidget);
	// END
	if (lCheckListBox==0){
		cout << "The passed value is not a CheckListBox" << endl;
		PL_fail;}
	// END integrity checks
	if(lCheckListBox->TheDateIsVisible())
		PL_succeed;
	else
		PL_fail;
}

foreign_t checkListBoxSetFoldersVisibility(term_t aCheckListBoxPrologName, 
					   // visible or hidden
					   term_t aVisible){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aCheckListBoxPrologName)];
	if (lWidget==0){
		cout << "checkListBoxSetDateVisibility: The passed value is not a Widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	SegCheckListBox * lCheckListBox = dynamic_cast<SegCheckListBox*> (lWidget);
	// END
	if (lCheckListBox==0){
		cout << "The passed value is not a CheckListBox" << endl;
		PL_fail;}
	// END integrity checks
	bool lVisible = (SegTerm::termToString(aVisible) == string("visible"));
	lCheckListBox->SetFoldersVisibility(lVisible);
	PL_succeed;
}

foreign_t checkListBoxTheDirIsVisible(term_t aCheckListBoxPrologName){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aCheckListBoxPrologName)];
	if (lWidget==0){
		cout << "checkListBoxSetDateVisibility: The passed value is not a Widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	SegCheckListBox * lCheckListBox = dynamic_cast<SegCheckListBox*> (lWidget);
	// END
	if (lCheckListBox==0){
		cout << "The passed value is not a CheckListBox" << endl;
		PL_fail;}
	// END integrity checks
	if(lCheckListBox->TheDirIsVisible())
		PL_succeed;
	else
		PL_fail;
}


foreign_t checkListBoxTheSizeIsVisible(term_t aCheckListBoxPrologName){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aCheckListBoxPrologName)];
	if (lWidget==0){
		cout << "checkListBoxSetDateVisibility: The passed value is not a Widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	SegCheckListBox * lCheckListBox = dynamic_cast<SegCheckListBox*> (lWidget);
	// END
	if (lCheckListBox==0){
		cout << "The passed value is not a CheckListBox" << endl;
		PL_fail;}
	// END integrity checks
	if(lCheckListBox->TheSizeIsVisible())
		PL_succeed;
	else
		PL_fail;
}

// set visibility of the date label, for ALL ITEMS in the CheckListBox.
foreign_t checkListBoxSetDateVisibility(term_t aCheckListBoxPrologName, 
					// 'hidden' or 'visible'
					term_t aVisibility){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aCheckListBoxPrologName)];
	if (lWidget==0){
		cout << "checkListBoxSetDateVisibility: The passed value is not a Widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	SegCheckListBox * lCheckListBox = dynamic_cast<SegCheckListBox*> (lWidget);
	// END
	if (lCheckListBox==0){
		cout << "The passed value is not a CheckListBox" << endl;
		PL_fail;}
	// END integrity checks
	bool lVisible = (SegTerm::termToString(aVisibility) == string("visible"));
	lCheckListBox->SetDateVisibility(lVisible);
	PL_succeed;
}
// set visibility of the dir label, for ALL ITEMS in the CheckListBox.
foreign_t checkListBoxSetDirVisibility(term_t aCheckListBoxPrologName, 
					// 'hidden' or 'visible'
					term_t aVisibility){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aCheckListBoxPrologName)];
	if (lWidget==0){
		cout << "checkListBoxSetDirVisibility: The passed value is not a Widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	SegCheckListBox * lCheckListBox = dynamic_cast<SegCheckListBox*> (lWidget);
	// END
	if (lCheckListBox==0){
		cout << "The passed value is not a CheckListBox" << endl;
		PL_fail;}
	// END integrity checks
	bool lVisible = (SegTerm::termToString(aVisibility) == string("visible"));
	lCheckListBox->SetDirVisibility(lVisible);
	PL_succeed;
}

foreign_t checkListBoxDeleteItem(term_t aCheckListBoxPrologName, 
			      term_t aItemPrologName){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aCheckListBoxPrologName)];
	if (lWidget==0){
		cout << "checkListBoxDeleteItem: The passed value is not a Widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	SegCheckListBox * lCheckListBox = dynamic_cast<SegCheckListBox*> (lWidget);
	// END
	if (lCheckListBox==0){
		cout << "The passed value is not a CheckListBox" << endl;
		PL_fail;}
	// END integrity checks
	lCheckListBox->DeleteItem(SegTerm(aItemPrologName));
	PL_succeed;
}



foreign_t windowMaximize(term_t aWindowPrologName){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aWindowPrologName)];
	if (lWidget==0){
		cout << "windowMaximize: The passed value is not a Widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	Window * l = dynamic_cast<Window*> (lWidget);
	// END
	if (l==0){
		cout << "The passed value is not a Window" << endl;
		PL_fail;}
	// END integrity checks
	l->maximize();
	PL_succeed;
}
foreign_t windowMaximizeVertically(term_t aWindowPrologName){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aWindowPrologName)];
	if (lWidget==0){
		cout << "windowMaximize: The passed value is not a Widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	Window * l = dynamic_cast<Window*> (lWidget);
	// END
	if (l==0){
		cout << "The passed value is not a Window" << endl;
		PL_fail;}
	// END integrity checks
	l->set_size_request(-1, l->get_screen()->get_height());
	PL_succeed;
}
foreign_t screenHeight(term_t aTestWindow, term_t aHeight){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aTestWindow)];
	if (lWidget==0){
		cout << "windowMaximize: The passed value is not a Widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	Window * l = dynamic_cast<Window*> (lWidget);
	// END
	if (l==0){
		cout << "The passed value is not a Window" << endl;
		PL_fail;}
	// END integrity checks
	int lHeight = l->get_screen()->get_height();
	// REGION unify aHeight with the height
	term_t lTempTerm = PL_new_term_ref();
	PL_put_integer(lTempTerm, lHeight);
	PL_unify(aHeight, lTempTerm);
	// END
	PL_succeed;
}
foreign_t windowCenter(term_t aTestWindow){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aTestWindow)];
	if (lWidget==0){
		cout << "windowMaximize: The passed value is not a Widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	Window * l = dynamic_cast<Window*> (lWidget);
	// END
	if (l==0){
		cout << "The passed value is not a Window" << endl;
		PL_fail;}
	// END integrity checks
	l->set_position(WIN_POS_CENTER);
	PL_succeed;
}
foreign_t widgetShow(term_t aWidgetPrologName){
	SegTerm lWidgetName(aWidgetPrologName);
	Widget * lWidget = gPrologNameToGtkWidget[lWidgetName];
	if (lWidget == 0){
		cout << "widgetShow: the passed value is not a Widget" << endl;
		throw string();	}
	if (!lWidget->is_visible()){
		lWidget->show();}
	PL_succeed;

}
foreign_t widgetQueueResize(term_t aWidgetPrologName){
	SegTerm lWidget(aWidgetPrologName);
	gPrologNameToGtkWidget[lWidget]->queue_resize();
	PL_succeed;
}
void string2int(const char* digit, int& result) {
   result = 0;

   //--- Convert each digit char and add into result.
   while (*digit >= '0' && *digit <='9') {
      result = (result * 10) + (*digit - '0');
      digit++;
   }

   //--- Check that there were no non-digits at end.
   if (*digit != 0) {
	   throw "bad string";
   }
}

foreign_t widgetSetSizeRequest(term_t aWidgetPrologName, term_t width, term_t height){
	SegTerm lWidget(aWidgetPrologName);
	int lWidth, lHeight;
	//string2int(SegTerm::termToString(width).c_str(), lWidth);
	//string2int(SegTerm::termToString(height).c_str(), lHeight);
	PL_get_integer(width, &lWidth);
	PL_get_integer(height, &lHeight);
	gPrologNameToGtkWidget[lWidget]->set_size_request(lWidth, lHeight);
	PL_succeed;
}
foreign_t widgetHide(term_t aWidgetPrologName){
	SegTerm lWidgetName(aWidgetPrologName);
	Widget * lWidget = gPrologNameToGtkWidget[lWidgetName];
	if (lWidget == 0){
		cout << "widgetHide: the passed value is not a Widget" << endl;
		throw string();	}
	if (lWidget->is_visible()){
		lWidget->hide();}
	PL_succeed;
}
foreign_t widgetDisable(term_t aWidgetPrologName){
	SegTerm lWidget(aWidgetPrologName);
	gPrologNameToGtkWidget[lWidget]->set_sensitive(false);
	PL_succeed;
}
foreign_t widgetEnable(term_t aWidgetPrologName){
	SegTerm lWidget(aWidgetPrologName);
	gPrologNameToGtkWidget[lWidget]->set_sensitive(true);
	PL_succeed;
}
foreign_t windowNew(term_t aWindowRef, term_t aTitle, /*file name*/term_t aIcon){

	SegWindow* lWindow = new SegWindow(shared_ptr<SegTerm>(new SegTerm(aWindowRef))); 
	lWindow->set_title(SegTerm::termToString(aTitle));
	lWindow->set_icon_from_file(SegTerm::termToString(aIcon));
	lWindow->set_position(WIN_POS_CENTER);
	PL_succeed;
}
foreign_t widgetDestroy(term_t aWidgetPrologName){
	Widget * l = gPrologNameToGtkWidget[SegTerm(aWidgetPrologName)];
	if (l==0){
		cout << "widgetDestroy: the argument is not a widget. "<<SegTerm::termToString(aWidgetPrologName) << endl;
		PL_fail;}

	Dialog * lDialog = dynamic_cast<Dialog*> (l);
	if (lDialog != 0){
		cout << "ERROR: you must not call widgetDestroy on a dialog. Use dialogClose/1 instead\n";
		throw "error";
	}
	// REGION delete the widgets and all its children.  This
	// also removes the widget from the global map, i.e. calls
	// gPrologNameToGtkWidget.erase(SegTerm(aWidgetPrologName));
	delete l;
	// END
	PL_succeed;
}
foreign_t hBoxNew(term_t aBoxPrologName){
	manage(new SegHBox(shared_ptr<SegTerm> (new SegTerm(aBoxPrologName))));
	PL_succeed;
}
// foreign_t menuNew(term_t aBoxPrologName){
// 	manage(new SegMenu(shared_ptr<SegTerm> (new SegTerm(aBoxPrologName))));
// 	PL_succeed;
// }
// foreign_t menuBarNew(term_t aBoxPrologName){
// 	manage(new SegMenuBar(shared_ptr<SegTerm> (new SegTerm(aBoxPrologName))));
// 	PL_succeed;
// }

// foreign_t menuCheckItemIsChecked(term_t aMenuPrologName, term_t aItemPrologName){
// 	// REGION integrity checks
// 	Widget* lWidget = 
// 		gPrologNameToGtkWidget[SegTerm(aMenuPrologName)];
// 	if (lWidget==0){
// 		cout << "menuAddItem: The passed value is not a widget" << endl;
// 		PL_fail;}
// 	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
// 	SegMenu * l = dynamic_cast<SegMenu*> (lWidget);
// 	// END
// 	if (l==0){
// 		cout << "The passed value is not a SegMenu" << endl;
// 		PL_fail;}
// 	// END integrity checks
// 	if (l->CheckItemIsPressed(shared_ptr<SegTerm>(new SegTerm(aItemPrologName)))) 
// 		PL_succeed;
// 	else
// 		PL_fail;
// }
// foreign_t menuAddItem(term_t aMenuPrologName, term_t aItemPrologName, term_t aItemText){
// 	// REGION integrity checks
// 	Widget* lWidget = 
// 		gPrologNameToGtkWidget[SegTerm(aMenuPrologName)];
// 	if (lWidget==0){
// 		cout << "menuAddItem: The passed value is not a widget" << endl;
// 		PL_fail;}
// 	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
// 	SegMenu * l = dynamic_cast<SegMenu*> (lWidget);
// 	// END
// 	if (l==0){
// 		cout << "The passed value is not a SegMenu" << endl;
// 		PL_fail;}
// 	// END integrity checks
// 	l->AddItem(shared_ptr<SegTerm>(new SegTerm(aItemPrologName)), 
// 		   SegTerm::termToString(aItemText));					    
							
// 	PL_succeed;
// }
// foreign_t menuAddCheckItem(term_t aMenuPrologName, term_t aItemPrologName, term_t aItemText){
// 	// REGION integrity checks
// 	Widget* lWidget = 
// 		gPrologNameToGtkWidget[SegTerm(aMenuPrologName)];
// 	if (lWidget==0){
// 		cout << "menuAddItem: The passed value is not a widget" << endl;
// 		PL_fail;}
// 	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
// 	SegMenu * l = dynamic_cast<SegMenu*> (lWidget);
// 	// END
// 	if (l==0){
// 		cout << "The passed value is not a SegMenu" << endl;
// 		PL_fail;}
// 	// END integrity checks
// 	l->AddCheckItem(shared_ptr<SegTerm>(new SegTerm(aItemPrologName)), 
// 		   SegTerm::termToString(aItemText));					    
							
// 	PL_succeed;
// }
// foreign_t menuBarAddMenu(term_t aMenuBarPrologName, term_t aMenuPrologName, term_t aMenuText){
// 	// REGION integrity checks
// 	Widget* lWidget = 
// 		gPrologNameToGtkWidget[SegTerm(aMenuBarPrologName)];
// 	if (lWidget==0){
// 		cout << "menuAddItem: The passed value is not a widget" << endl;
// 		PL_fail;}
// 	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
// 	SegMenuBar * l = dynamic_cast<SegMenuBar*> (lWidget);
// 	// END
// 	if (l==0){
// 		cout << "The passed value is not a SegMenuBar" << endl;
// 		PL_fail;}


// 	Widget * lW2 = 
// 		gPrologNameToGtkWidget[SegTerm(aMenuPrologName)];
// 	if (lW2==0){
// 		cout << "menuAddItem: The second passed value is not a widget" << endl;
// 		PL_fail;}
// 	SegMenu * l2 = dynamic_cast<SegMenu*> (lW2);
// 	if (l2==0){
// 		cout << "The second passed value is not a SegMenu" << endl;
// 		PL_fail;}
// 	// END integrity checks
// 	l->items().push_back(Gtk::Menu_Helpers::MenuElem(
// 					     SegTerm::termToString(aMenuText),
// 					     *l2));
// 	PL_succeed;
// }

foreign_t vBoxNew(term_t aBoxPrologName){
	manage(new SegVBox(shared_ptr<SegTerm>(new SegTerm(aBoxPrologName))));
	PL_succeed;
}
foreign_t containerAdd(term_t aContainerPrologName, term_t aWidgetPrologName){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aContainerPrologName)];
	if (lWidget==0){
		cout << "containerAdd: The passed value is not a widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	Container * lContainerPtr = dynamic_cast<Container*> (lWidget);
	// END
	if (lContainerPtr==0){
		cout << "The passed value is not a container" << endl;
		PL_fail;}
	// END integrity checks
	lContainerPtr->add(*gPrologNameToGtkWidget[SegTerm(aWidgetPrologName)]);
	PL_succeed;
}
foreign_t boxPackStart4(term_t aBoxPrologName, term_t aWidgetPrologName, term_t aExpand, term_t aFill){
	bool lExpand = (SegTerm::termToString(aExpand) == "expand");
	bool lFill = (SegTerm::termToString(aFill) == "fill");
	// REGION integrity checks
	Widget* lBoxWidget = 
		gPrologNameToGtkWidget[SegTerm(aBoxPrologName)];
	if (lBoxWidget==0){
		cout << "The passed box is not a widget" << endl;
		PL_fail;}
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aWidgetPrologName)];
	if (lWidget==0){
		cout << "The passed widget is not a widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	Box * lBoxPtr = dynamic_cast<Box*> (lBoxWidget);
	// END
	if (lBoxPtr==0){
		cout << "The first arg is not a box" << endl;
		PL_fail;}
	// END integrity checks
	lBoxPtr->pack_start(*lWidget, lExpand, lFill);
	PL_succeed;
}
foreign_t boxPackStart5(term_t aBoxPrologName, term_t aWidgetPrologName, term_t aExpand, term_t aFill, term_t aPadding){
	// REGION integrity checks
	Widget* lBoxWidget = 
		gPrologNameToGtkWidget[SegTerm(aBoxPrologName)];
	if (lBoxWidget==0){
		cout << "The passed box is not a widget" << endl;
		PL_fail;}
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aWidgetPrologName)];
	if (lWidget==0){
		cout << "The passed widget is not a widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	Box * lBoxPtr = dynamic_cast<Box*> (lBoxWidget);
	// END
	if (lBoxPtr==0){
		cout << "The first arg is not a box" << endl;
		PL_fail;}
	// END integrity checks

	bool lExpand = (SegTerm::termToString(aExpand) == "expand");
	bool lFill = (SegTerm::termToString(aFill) == "fill");
	int lPadding;
	PL_get_integer(aPadding, &lPadding);
	lBoxPtr->pack_start(*lWidget, lExpand, lFill, lPadding);
	PL_succeed;
}
foreign_t boxPackEnd4(term_t aBoxPrologName, term_t aWidgetPrologName, term_t aExpand, term_t aFill){
	bool lExpand = (SegTerm::termToString(aExpand) == "expand");
	bool lFill = (SegTerm::termToString(aFill) == "fill");
	// REGION integrity checks
	Widget* lBoxWidget = 
		gPrologNameToGtkWidget[SegTerm(aBoxPrologName)];
	if (lBoxWidget==0){
		cout << "The passed box is not a widget" << endl;
		PL_fail;}
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aWidgetPrologName)];
	if (lWidget==0){
		cout << "The passed widget is not a widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	Box * lBoxPtr = dynamic_cast<Box*> (lBoxWidget);
	// END
	if (lBoxPtr==0){
		cout << "The first arg is not a box" << endl;
		PL_fail;}
	// END integrity checks
	lBoxPtr->pack_end(*lWidget, lExpand, lFill);
	PL_succeed;
}
foreign_t boxPackEnd5(term_t aBoxPrologName, term_t aWidgetPrologName, term_t aExpand, term_t aFill, term_t aPadding){
	bool lExpand = (SegTerm::termToString(aExpand) == "expand");
	bool lFill = (SegTerm::termToString(aFill) == "fill");
	// REGION integrity checks
	Widget* lBoxWidget = 
		gPrologNameToGtkWidget[SegTerm(aBoxPrologName)];
	if (lBoxWidget==0){
		cout << "The passed box is not a widget" << endl;
		PL_fail;}
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aWidgetPrologName)];
	if (lWidget==0){
		cout << "The passed widget is not a widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	Box * lBoxPtr = dynamic_cast<Box*> (lBoxWidget);
	// END
	if (lBoxPtr==0){
		cout << "The first arg is not a box" << endl;
		PL_fail;}
	// END integrity checks
	int lPadding;
	PL_get_integer(aPadding, &lPadding);
	lBoxPtr->pack_start(*lWidget, lExpand, lFill, lPadding);

	lBoxPtr->pack_end(*lWidget, lExpand, lFill, lPadding);
	PL_succeed;
}

foreign_t labelNew(term_t aRef, term_t aText, term_t aLineWrap){
	string lText = SegTerm::termToString(aText);
	SegLabel* lLabel = manage(new SegLabel(shared_ptr<SegTerm>(new SegTerm(aRef)), lText));
	if (SegTerm::termToString(aLineWrap)=="lineWrap"){
		lLabel->set_line_wrap(true);}
	lLabel->set_padding(12, 9);
	PL_succeed;
}

/// create a label  which can have a tooltip.
foreign_t eventLabelNew(term_t aRef, term_t aText, term_t aLineWrap){
	string lText = SegTerm::termToString(aText);
	bool lLineWrap = (SegTerm::termToString(aLineWrap)=="lineWrap");
	SegEventLabel* lLabel = manage(new SegEventLabel(shared_ptr<SegTerm>(new SegTerm(aRef)), lText, lLineWrap));
	PL_succeed;
}

foreign_t eventBoxNew(term_t aRef){
	manage(new SegEventBox(shared_ptr<SegTerm>(new SegTerm(aRef))));
	PL_succeed;
}

foreign_t buttonNew(term_t aPrologName){
	manage(new SegButton(shared_ptr<SegTerm>(new SegTerm(aPrologName))));
	PL_succeed;
}
foreign_t buttonNewWithLabel(term_t aPrologName, term_t aText){
	SegButton * lButton = manage(new SegButton(shared_ptr<SegTerm>(new SegTerm(aPrologName))));
	Label * lLabel = manage(new Label(SegTerm::termToString(aText)));
	lLabel->show();
	lLabel->set_padding(9,6); // @@ hardcoded
	lButton->add(*lLabel);
	PL_succeed;
}

foreign_t entryNew(term_t aRef){
	manage(new SegEntry(shared_ptr<SegTerm>(new SegTerm(aRef))));
	PL_succeed;
}

foreign_t entryGetText(term_t aTextBox, term_t aText){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aTextBox)];
	if (lWidget==0){
		cout << "entryGetText: The passed value is not a widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	Entry * lEntryPtr = dynamic_cast<Entry*> (lWidget);
	// END
	if (lEntryPtr==0){
		cout << "The passed value is not an entry" << endl;
		PL_fail;}
	// END integrity checks
	// REGION unify aText with the text
	term_t lTextTerm = PL_new_term_ref();
	PL_put_atom_chars(lTextTerm, lEntryPtr->get_text().c_str());
	PL_unify(aText, lTextTerm);
	// END
	PL_succeed;
}
foreign_t entrySetText(term_t aTextBox, term_t aText){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aTextBox)];
	if (lWidget==0){
		cout << "entrySetText: The passed value is not a widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	Entry * lEntryPtr = dynamic_cast<Entry*> (lWidget);
	// END
	if (lEntryPtr==0){
		cout << "The passed value is not an entry" << endl;
		PL_fail;}
	// END integrity checks
	// REGION set text
	lEntryPtr->set_text(SegTerm::termToString(aText));
	// END
	PL_succeed;
}


foreign_t labelSetText(term_t aRef, term_t aText){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aRef)];
	if (lWidget==0){
		cout << "The passed value is not a widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	Label * lLabelPtr = dynamic_cast<Label*> (lWidget);
	// END
	if (lLabelPtr==0){
		cout << "The passed value is not a label" << endl;
		PL_fail;}
	// END integrity checks
	lLabelPtr->set_text(SegTerm::termToString(aText));
	PL_succeed;
}
foreign_t eventLabelSetText(term_t aRef, term_t aText){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aRef)];
	if (lWidget==0){
		cout << "The passed value is not a widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	SegEventLabel * lLabelPtr = dynamic_cast<SegEventLabel*> (lWidget);
	// END
	if (lLabelPtr==0){
		cout << "The passed value is not a label" << endl;
		PL_fail;}
	// END integrity checks
	lLabelPtr->SetText(SegTerm::termToString(aText));
	PL_succeed;
}
foreign_t buttonSetText(term_t aRef, term_t aText){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aRef)];
	if (lWidget==0){
		cout << "The passed value is not a widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	Button * l = dynamic_cast<Button*> (lWidget);
	// END
	if (l==0){
		cout << "The passed value is not a button" << endl;
		PL_fail;}
	// END integrity checks
	l->set_label(SegTerm::termToString(aText));
	PL_succeed;
}
foreign_t dialogClose(term_t aRef){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aRef)];
	if (lWidget==0){
		cout << "The passed value is not a widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	SegDialog * l = dynamic_cast<SegDialog*> (lWidget);
	// END
	if (l==0){
		cout << "The passed value is not a SegDialog" << endl;
		PL_fail;}
	// END integrity checks
	l->Close();
	PL_succeed;
}

foreign_t toggleButtonIsPressed(term_t aRef){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aRef)];
	if (lWidget==0){
		cout << "The passed value is not a widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	ToggleButton * l = dynamic_cast<ToggleButton*> (lWidget);
	// END
	if (l==0){
		cout << "The passed value is not a ToggleButton" << endl;
		PL_fail;}
	// END integrity checks
	bool pressed = l->get_active();
	if (pressed){
		PL_succeed;
	}
	else{
		PL_fail;
	}
}
foreign_t miscSetAlignment(term_t aRef, term_t aAlignmentX, term_t aAlignmentY){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aRef)];
	if (lWidget==0){
		cout << "The passed value is not a widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	Misc * l = dynamic_cast<Misc*> (lWidget);
	// END
	if (l==0){
		cout << "The passed value is not a Misc" << endl;
		PL_fail;}
	// END integrity checks
	double lAlignX ;
	PL_get_float(aAlignmentX, &lAlignX);
	double lAlignY;
	PL_get_float(aAlignmentY, &lAlignY);
	l->set_alignment(lAlignX, lAlignY);
	PL_succeed;

}
foreign_t miscSetPadding(term_t aRef, term_t aAlignmentX, term_t aAlignmentY){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aRef)];
	if (lWidget==0){
		cout << "The passed value is not a widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	Misc * l = dynamic_cast<Misc*> (lWidget);
	// END
	if (l==0){
		cout << "The passed value is not a Misc" << endl;
		PL_fail;}
	// END integrity checks
	int lAlignX ;
	PL_get_integer(aAlignmentX, &lAlignX);
	int lAlignY;
	PL_get_integer(aAlignmentY, &lAlignY);
	l->set_padding(lAlignX, lAlignY);
	PL_succeed;

}
foreign_t toggleButtonSetPressedState(term_t aRef, 
					   term_t aState/* pressed or released*/){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aRef)];
	if (lWidget==0){
		cout << "The passed value is not a widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	ToggleButton * l = dynamic_cast<ToggleButton*> (lWidget);
	// END
	if (l==0){
		cout << "The passed value is not a ToggleButton" << endl;
		PL_fail;}
	// END integrity checks
	bool lPressedState = (SegTerm::termToString(aState) == string("pressed"));
	if (lPressedState)
		l->set_active(true);
	else
		l->set_active(false);
	PL_succeed;

}
foreign_t buttonSetReliefNone(term_t aRef){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aRef)];
	if (lWidget==0){
		cout << "The passed value is not a widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	Button * l = dynamic_cast<Button*> (lWidget);
	// END
	if (l==0){
		cout << "The passed value is not a Button" << endl;
		PL_fail;}
	// END integrity checks
	l->set_relief(RELIEF_NONE);
	PL_succeed;

}
foreign_t checkListBoxGetOptimalHeight(term_t aRef, term_t aHeight){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aRef)];
	if (lWidget==0){
		cout << "checkListBoxGetOptimalHeight: The passed value is not a Widget: " << SegTerm::termToString(aRef) << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	SegCheckListBox * l = dynamic_cast<SegCheckListBox*> (lWidget);
	// END
	if (l==0){
		cout << "checkListBoxGetOptimalHeight: The passed value is not a SegCheckListBox" << endl;
		PL_fail;}
	// END integrity checks
	int lHeight = l->OptimalHeight();
	// REGION unify aHeight with the height
	term_t lTempTerm = PL_new_term_ref();
	PL_put_integer(lTempTerm, lHeight);
	PL_unify(aHeight, lTempTerm);
	// END
	
	PL_succeed;

}
foreign_t checkListBoxSetScrollPositionToTheTop(term_t aRef){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aRef)];
	if (lWidget==0){
		cout << "checkListBoxGetOptimalHeight: The passed value is not a Widget: " << SegTerm::termToString(aRef) << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	SegCheckListBox * l = dynamic_cast<SegCheckListBox*> (lWidget);
	// END
	if (l==0){
		cout << "checkListBoxGetOptimalHeight: The passed value is not a SegCheckListBox" << endl;
		PL_fail;}
	// END integrity checks
	l->SetScrollPositionToTheTop();
	PL_succeed;
}
foreign_t checkListBoxDefeatBug(term_t aRef){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aRef)];
	if (lWidget==0){
		cout << "checkListBoxGetOptimalHeight: The passed value is not a Widget: " << SegTerm::termToString(aRef) << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	SegCheckListBox * l = dynamic_cast<SegCheckListBox*> (lWidget);
	// END
	if (l==0){
		cout << "checkListBoxGetOptimalHeight: The passed value is not a SegCheckListBox" << endl;
		PL_fail;}
	// END integrity checks
	l->DefeatBug();
	PL_succeed;
}


foreign_t imageButtonNew(term_t aRef, term_t aImageFile, term_t aText){
	manage(	new SegImageButton(shared_ptr<SegTerm>(new SegTerm(aRef)), 
				   SegTerm::termToString(aImageFile),
				   SegTerm::termToString(aText)));
 	PL_succeed;
}
foreign_t toggleButtonNew(term_t aRef, term_t aText){
	manage(	new SegToggleButton(shared_ptr<SegTerm>(new SegTerm(aRef)), 
				    SegTerm::termToString(aText)));
 	PL_succeed;
}
foreign_t imageNew(term_t aRef, term_t aImageFile){
	manage(	new SegImage(shared_ptr<SegTerm>(new SegTerm(aRef)), 
			     SegTerm::termToString(aImageFile)));
 	PL_succeed;
}
foreign_t textViewNew(term_t aRef){
	cout << "textViewNew" << endl;
	manage(new SegTextView(shared_ptr<SegTerm>(new SegTerm(aRef))));
	
 	PL_succeed;
}
foreign_t textViewAddLine(term_t aRef, term_t aLine){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aRef)];
	if (lWidget==0){
		cout << "The passed value is not a widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	SegTextView * l = dynamic_cast<SegTextView*> (lWidget);
	// END
	if (l==0){
		cout << "The passed value is not a TextView" << endl;
		PL_fail;}
	// END integrity checks
	l->AddLine(SegTerm::termToString(aLine));
 	PL_succeed;
}
// foreign_t optionMenuSetMenu(term_t aRef, term_t aMenu){
// 	// REGION integrity checks
// 	Widget* lWidget = 
// 		gPrologNameToGtkWidget[SegTerm(aRef)];
// 	if (lWidget==0){
// 		cout << "The passed value is not a widget" << endl;
// 		PL_fail;}
// 	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
// 	OptionMenu * l = dynamic_cast<OptionMenu*> (lWidget);
// 	// END
// 	if (l==0){
// 		cout << "The passed value is not an OptionMenu" << endl;
// 		PL_fail;}
// 	// END integrity checks


// 	// REGION integrity checks
// 	Widget* lWidget2 = 
// 		gPrologNameToGtkWidget[SegTerm(aMenu)];
// 	if (lWidget==0){
// 		cout << "The 2nd passed value is not a widget" << endl;
// 		PL_fail;}
// 	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
// 	Menu * l2 = dynamic_cast<Menu*> (lWidget2);
// 	// END
// 	if (l2==0){
// 		cout << "The passed value is not an Menu" << endl;
// 		PL_fail;}
// 	// END integrity checks

// 	l->set_menu(*l2);
//  	PL_succeed;
// }

foreign_t optionMenuAddItem(term_t aRef, term_t aItemData, term_t aItemName){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aRef)];
	if (lWidget==0){
		cout << "The passed value is not a widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	SegOptionMenu * l = dynamic_cast<SegOptionMenu*> (lWidget);
	// END
	if (l==0){
		cout << "The passed value is not an OptionMenu" << endl;
		PL_fail;}
	// END integrity checks
	l->AddItem(shared_ptr<SegTerm>(new SegTerm(aItemData)), SegTerm::termToString(aItemName));
 	PL_succeed;
}
foreign_t optionMenuSetSelectedItem(term_t aRef, term_t aItemData){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aRef)];
	if (lWidget==0){
		cout << "The passed value is not a widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	SegOptionMenu * l = dynamic_cast<SegOptionMenu*> (lWidget);
	// END
	if (l==0){
		cout << "The passed value is not an OptionMenu" << endl;
		PL_fail;}
	// END integrity checks
	l->SetSelectedItem(shared_ptr<SegTerm>(new SegTerm(aItemData)));
 	PL_succeed;
}

// foreign_t optionMenuGetHistory(term_t aRef, term_t a){
// 	// REGION integrity checks
// 	Widget* lWidget = 
// 		gPrologNameToGtkWidget[SegTerm(aRef)];
// 	if (lWidget==0){
// 		cout << "The passed value is not a widget" << endl;
// 		PL_fail;}
// 	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
// 	OptionMenu * l = dynamic_cast<OptionMenu*> (lWidget);
// 	// END
// 	if (l==0){
// 		cout << "The passed value is not an OptionMenu" << endl;
// 		PL_fail;}
// 	// END integrity checks

// 	int lHistory = l->get_history();
// 	// REGION unify a with lHistory
// 	term_t lTempTerm = PL_new_term_ref();
// 	PL_put_integer(lTempTerm, lHistory);
// 	PL_unify(a, lTempTerm);
// 	// END

//  	PL_succeed;
// }


foreign_t imageToggleButtonNew(term_t aRef, term_t aImageFile, term_t aText){
	SegImageToggleButton* l = manage(	new SegImageToggleButton(
								shared_ptr<SegTerm>(new SegTerm(aRef)), 
								SegTerm::termToString(aImageFile),
								SegTerm::termToString(aText)));
// 	if (gTooltips!=0)
// 		gTooltips->set_tip(*l, "toggle button\nbla bla");
// 	else
// 		cout <<"gTooltips = 0"<<endl;
 	PL_succeed;
}
// foreign_t toolbarNew(term_t aRef){
// 	manage(	new SegToolbar(	shared_ptr<SegTerm>(new SegTerm(aRef))));
//  	PL_succeed;
// }
foreign_t widgetSetTooltip(term_t aRef, term_t aText){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aRef)];
	if (lWidget==0){
		cout << "The passed value is not a widget" << endl;
		PL_fail;}
	// END integrity checks
	gTooltips->set_tip(*lWidget, SegTerm::termToString(aText));
 	PL_succeed;
}
foreign_t widgetIsVisible(term_t aRef){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aRef)];
	if (lWidget==0){
		cout << "The passed value is not a widget" << endl;
		PL_fail;}
	// END integrity checks
	if (lWidget->is_visible())
		PL_succeed;
	else
		PL_fail;
}


foreign_t fileOrDirSelection(/*in*/term_t aMessage, /*out*/term_t aChosenFile ){
	FileSelection f(SegTerm::termToString(aMessage));
	f.set_filename("~/");
	f.maximize();
	Label * lLabel = manage(new Label(SegTerm::termToString(aMessage)));
	lLabel->show();
	f.get_main_vbox()->pack_start(*lLabel, false, true, 1);
	f.run();
	// REGION unify aChosenFile with the selected file/dir
	term_t lFileName = PL_new_term_ref();
	PL_put_atom_chars(lFileName, f.get_filename().c_str());
	PL_unify(aChosenFile, lFileName);
	// END
	PL_succeed;
}

foreign_t checkListBoxNew(term_t aRef, 
			  term_t aRelief, 
			  term_t aReadableRepresentationOfWhatIAmHolding,
			  // selectedListVisible or selectedListHidden
			  term_t aShowTheSelectedList){
	bool lRelief = (SegTerm::termToString(aRelief) == string("relief"));
	bool lShowTheSelectedList = (SegTerm::termToString(aRelief) == string("selectedListVisible"));
	manage(new SegCheckListBox(shared_ptr<SegTerm>(new SegTerm(aRef)), lRelief, 
				   SegTerm::termToString(aReadableRepresentationOfWhatIAmHolding),
				   lShowTheSelectedList)); 

 	PL_succeed;}
foreign_t hSeparatorNew(term_t aRef){
	manage(new SegHSeparator(shared_ptr<SegTerm>(new SegTerm(aRef)))); 
 	PL_succeed;}
foreign_t vSeparatorNew(term_t aRef){
	manage(new SegVSeparator(shared_ptr<SegTerm>(new SegTerm(aRef)))); 
 	PL_succeed;}

foreign_t optionMenuNew(term_t aRef){
	manage(new SegOptionMenu(shared_ptr<SegTerm>(new SegTerm(aRef)))); 
 	PL_succeed;}
foreign_t scrolledWindowNew(term_t aRef){
	manage(new SegScrolledWindow(shared_ptr<SegTerm>(new SegTerm(aRef)))); 
 	PL_succeed;}

foreign_t checkListBoxSetKeepScrollingBoxesAlwaysVisible(term_t aCheckListBoxPrologName, 
					    term_t aState/* disabled or enabled*/){
        // REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aCheckListBoxPrologName)];
	if (lWidget==0){
		cout << "The passed value is not a Widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	SegCheckListBox * lCheckListBox = dynamic_cast<SegCheckListBox*> (lWidget);
	// END
	if (lCheckListBox==0){
		cout << "The passed value is not a CheckListBox" << endl;
		PL_fail;}
	// END integrity checks

 	if (SegTerm::termToString(aState) == "enabled")
 	        lCheckListBox->SetKeepScrollingBoxesAlwaysVisible(true);
 	else
 	        lCheckListBox->SetKeepScrollingBoxesAlwaysVisible(false);

	PL_succeed;
}


foreign_t checkListBoxAddItem(term_t aCheckListBoxPrologName, 
			      // any term.
			      term_t aItemPrologName,
			      // usually a string with newlines
			      term_t aItemText, 
			      // can be '' for no tooltip
			      term_t aTooltip,
			      // typically a list of atoms: [book, good, 'Michael Jackson'].
			      term_t aFolders,
			      // can be ''
			      term_t aDirText, 
			      //can be ''
			      term_t aSize,
			      // can be ''
			      term_t aDate,
			      // can be ''
			      term_t aImage  ){
        // REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aCheckListBoxPrologName)];
	if (lWidget==0){
		cout << "The passed value is not a Widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	SegCheckListBox * lCheckListBox = dynamic_cast<SegCheckListBox*> (lWidget);
	// END
	if (lCheckListBox==0){
		cout << "The passed value is not a CheckListBox" << endl;
		PL_fail;}
	// END integrity checks
	lCheckListBox->AddItem(shared_ptr<SegTerm>(new SegTerm(aItemPrologName)), 
			       aItemText,
			       lCheckListBox->get_pango_context(),
			       SegTerm::termToString(aTooltip),
			       SegTerm::termToString(aFolders),
			       SegTerm::termToString(aDirText),
			       SegTerm::termToString(aSize),
			       SegTerm::termToString(aDate),
			       SegTerm::termToString(aImage));
	PL_succeed;}

foreign_t theFileHasAValidUtf8Name(term_t aName){
	char * lString;
	int lRet = PL_get_atom_chars(aName, &lString);
	if (lRet ==FALSE){
		lRet = PL_get_chars(aName, &lString, CVT_ALL | CVT_WRITE);
		if (lRet==FALSE){
			cout << "theFileHasAValidUtf8Name: error in PL_get_chars"<<endl;
			throw string();}
		else{
			cout << "theFileHasAValidUtf8Name: coding error. You probably did not pass an atom: "<<lString << endl;
			throw string();}}
	ustring lStringUtf(lString);
	if (lStringUtf.validate()){
		PL_succeed;}
	else{
		PL_fail;}}
foreign_t splitStringIntoWords(term_t aStringToBeSplit /*in : must be an atom */, 
			       term_t oWords /*out: a list of atoms */){
	char * lStringToBeSplit;
	int lReturn = PL_get_atom_chars(/*in*/aStringToBeSplit, &lStringToBeSplit/*out*/);
	if (lReturn==FALSE){
		cout << "splitStringIntoWords: PL_get_atom_chars failed" << endl; 
		throw string("PL_get_atom_chars failed");}
	ustring lStringToBeSplitUtf(lStringToBeSplit);
	std::set<ustring> lWords = gSplitStringIntoWords(lStringToBeSplitUtf);
	// REGION create a prolog list from lWords
	term_t lTempAtom = PL_new_term_ref();
	term_t lOutputList = PL_new_term_ref();
	PL_put_nil(lOutputList);
	std::set<ustring>::const_iterator i;
	for (i=lWords.begin(); i!= lWords.end(); i++){
		PL_put_atom_chars(lTempAtom, (*i).c_str());
		PL_cons_list(lOutputList, lTempAtom, lOutputList);}
	// END
	PL_unify(oWords, lOutputList);
	PL_succeed;}

foreign_t checkListBoxSetItemFolders(term_t aCheckListBoxPrologName, 
			      // any term.
			      term_t aItemPrologName,
			      // typically a list of atoms: [book, good, 'Michael Jackson'].
			      term_t aFolders){
	// REGION integrity checks
	Widget* lWidget = 
		gPrologNameToGtkWidget[SegTerm(aCheckListBoxPrologName)];
	if (lWidget==0){
		cout << "The passed value is not a Widget" << endl;
		PL_fail;}
	// REGION I cannot dynamic_cast a smart ptr, so I extract the ptr
	SegCheckListBox * lCheckListBox = dynamic_cast<SegCheckListBox*> (lWidget);
	// END
	if (lCheckListBox==0){
		cout << "The passed value is not a CheckListBox" << endl;
		PL_fail;}
	// END integrity checks
	lCheckListBox->SetItemFolders(shared_ptr<SegTerm>(new SegTerm(aItemPrologName)), 
				      SegTerm::termToString(aFolders));
	PL_succeed;}






/// REGION this is global because it is referred to in
/// sleepUntilEventIsRaised/1
shared_ptr<Gtk::Main> gGtkMain;
// END

bool theCallbackhasBeenCalled(){
	return gLatestEvents.size()>0; //PL_is_compound(gLatestEvent) || PL_is_atom(gLatestEvent);
}
foreign_t sleepUntilEventIsRaised(/* out*/term_t oEvent){
	while(gLatestEvents.size()==0){
		gGtkMain->iteration(true); // true = blocking. 
		// If we are here, AT LEAST one gtk event was
		// generated, and ZERO OR MORE callbacks I defined
		// have been called. 
	} 

	// if we are here, then AT LEAST one event is in the queue. We
	// pick the first event from the list gLatestEvents, and unify
	// oEvent with it.

	// REGION if the event is not "timer", output to stdout
	//string lEvent = gLatestEvents.begin()->ToString(); maybe it is this that leaks?
	//if (lEvent != "timer")
	//	cout << "sleepUntilEventIsRaised: raising event " << lEvent << endl;
	// END
	//cout << "sleepUntilEventIsRaised: event queue size = " << gLatestEvents.size() << endl;
	PL_unify(oEvent, gLatestEvents.begin()->ToPrologTerm());
	gLatestEvents.pop_front();
	PL_succeed; 
}



foreign_t setTooltipsState(term_t aState/* disabled or enabled*/){
	if (SegTerm::termToString(aState) == "enabled")
		gTooltips->enable();
	else
		gTooltips->disable();

	PL_succeed;
}

// REGION declare the predicatres written in C
PL_extension predicates[] = {
	{"labelSetText", 2, (void*)labelSetText, 0},
	{"buttonSetText", 2, (void*)buttonSetText, 0},
	{"buttonSetReliefNone", 1, (void*)buttonSetReliefNone, 0},
	{"entryGetText", 2, (void*)entryGetText, 0},

	{"optionMenuNew", 1, (void*)optionMenuNew, 0},
 	{"optionMenuAddItem", 3, (void*)optionMenuAddItem, 0},
 	{"optionMenuSetSelectedItem", 2, (void*)optionMenuSetSelectedItem, 0},
// 	{"optionMenuSetMenu", 2, (void*)optionMenuSetMenu, 0},
// 	{"optionMenuSetHistory", 2, (void*)optionMenuSetHistory, 0},
// 	{"optionMenuGetHistory", 2, (void*)optionMenuGetHistory, 0},

	{"widgetSetTooltip", 2, (void*)widgetSetTooltip, 0},
	{"checkListBoxItemExists", 2, (void*)checkListBoxItemExists, 0},
	{"entrySetText", 2, (void*)entrySetText, 0},
	{"eventLabelNew", 3, (void*)eventLabelNew, 0},
	{"labelNew", 3, (void*)labelNew, 0},
	{"eventLabelSetText", 2, (void*)eventLabelSetText, 0},
	{"widgetSetSizeRequest", 3, (void*)widgetSetSizeRequest, 0},
	{"windowNew", 3, (void*)windowNew, 0},
	{"checkListBoxSetItemFolders", 3, (void*)checkListBoxSetItemFolders, 0},
	{"imageButtonNew", 3, (void*)imageButtonNew, 0},
	{"imageToggleButtonNew", 3, (void*)imageToggleButtonNew, 0},
	{"toggleButtonNew", 2, (void*)toggleButtonNew, 0},
	//	{"toolbarAddToggleButton", 5, (void*)toolbarAddToggleButton, 0},
	//{"toolbarAddButton", 5, (void*)toolbarAddButton, 0},
	//{"toolbarAddSpace", 1, (void*)toolbarAddSpace, 0},
	//{"toolbarAddLabel",2 , (void*)toolbarAddLabel, 0},
	{"entryNew", 1, (void*)entryNew, 0},
	{"checkListBoxDefeatBug", 1, (void*)checkListBoxDefeatBug, 0},
	{"checkListBoxTheDateIsVisible", 1, (void*)checkListBoxTheDateIsVisible, 0},
	{"checkListBoxTheSizeIsVisible", 1, (void*)checkListBoxTheSizeIsVisible, 0},
	{"checkListBoxTheDirIsVisible", 1, (void*)checkListBoxTheDirIsVisible, 0},
	{"widgetIsVisible", 1, (void*)widgetIsVisible, 0},
	{"eventBoxNew", 1, (void*)eventBoxNew, 0},
	{"handleBoxNew", 1, (void*)handleBoxNew, 0},
	{"frameNew", 1, (void*)frameNew, 0},
	{"setTooltipsState", 1, (void*)setTooltipsState, 0},
	{"checkListBoxSetKeepScrollingBoxesAlwaysVisible", 2, (void*)checkListBoxSetKeepScrollingBoxesAlwaysVisible, 0},
	{"textViewNew", 1, (void*)textViewNew, 0},
	//{"toolbarNew", 1, (void*)toolbarNew, 0},
	{"textViewAddLine", 2, (void*)textViewAddLine, 0},
	{"checkListBoxGetOptimalHeight", 2, (void*)checkListBoxGetOptimalHeight, 0},
	{"checkListBoxHideAllItems", 1, (void*)checkListBoxHideAllItems, 0},
	{"checkListBoxAddItem", 9, (void*)checkListBoxAddItem, 0},
	{"checkListBoxSetItemState", 3, (void*)checkListBoxSetItemState, 0},
	{"checkListBoxSetDirVisibility", 2, (void*)checkListBoxSetDirVisibility, 0},
	{"checkListBoxSetSizeVisibility", 2, (void*)checkListBoxSetSizeVisibility, 0},
	{"checkListBoxSetDateVisibility", 2, (void*)checkListBoxSetDateVisibility, 0},
	{"checkListBoxNew", 4, (void*)checkListBoxNew, 0},
	{"buttonNew", 1, (void*)buttonNew, 0},
	{"dialogClose", 1, (void*)dialogClose, 0},
	{"buttonNew", 2, (void*)buttonNewWithLabel, 0},
	{"imageNew", 2, (void*)imageNew, 0},
	{"fileOrDirSelection", 2, (void*)fileOrDirSelection, 0},
	{"checkListBoxDeleteItem", 2, (void*)checkListBoxDeleteItem, 0},
	{"boxPackStart", 4, (void*)boxPackStart4, 0},
	{"boxPackStart", 5, (void*)boxPackStart5, 0},
	{"boxPackEnd", 4, (void*)boxPackEnd4, 0},
	{"boxPackEnd", 5, (void*)boxPackEnd5, 0},
	{"containerAdd", 2, (void*)containerAdd, 0},
	{"imageSet", 2, (void*)imageSet, 0},
	{"dialogAddToVBox", 2, (void*)dialogAddToVBox, 0},
	{"toggleButtonSetPressedState", 2, (void*)toggleButtonSetPressedState, 0},
	{"vBoxNew", 1, (void*) vBoxNew, 0},
	{"toggleButtonIsPressed", 1, (void*) toggleButtonIsPressed, 0},
	{"hBoxNew", 1, (void*) hBoxNew, 0},
	{"widgetDestroy", 1, (void*) widgetDestroy, 0},
	{"widgetEnable", 1, (void*) widgetEnable, 0},
	{"widgetDisable", 1, (void*) widgetDisable, 0},
	{"widgetHide", 1, (void*) widgetHide, 0},
	{"checkListBoxClear", 1, (void*) checkListBoxClear, 0},
	{"checkListBoxSortByName", 1, (void*) checkListBoxSortByName, 0},
	{"checkListBoxAddSeparator", 1, (void*) checkListBoxAddSeparator, 0},
	{"checkListBoxSortBySize", 1, (void*) checkListBoxSortBySize, 0},
	{"checkListBoxSortByDate", 1, (void*) checkListBoxSortByDate, 0},
	{"checkListBoxSetFoldersVisibility", 2, (void*) checkListBoxSetFoldersVisibility, 0},
	{"widgetShow", 1, (void*) widgetShow, 0},
	{"scrolledWindowNew", 1, (void*) scrolledWindowNew, 0},
	{"widgetQueueResize", 1, (void*) widgetQueueResize, 0},
// 	{"menuNew", 1, (void*) menuNew, 0},
// 	{"menuBarNew", 1, (void*) menuBarNew, 0},
// 	{"menuCheckItemIsChecked", 2, (void*) menuCheckItemIsChecked, 0},
	{"screenHeight", 2, (void*) screenHeight, 0},
// 	{"menuAddItem", 3, (void*) menuAddItem, 0},
// 	{"menuAddCheckItem", 3, (void*) menuAddCheckItem, 0},
// 	{"menuBarAddMenu", 3, (void*) menuBarAddMenu, 0},
	{"hSeparatorNew", 1, (void*) hSeparatorNew, 0},
	{"checkListBoxSetScrollPositionToTheTop", 1, (void*) checkListBoxSetScrollPositionToTheTop, 0},
	{"vSeparatorNew", 1, (void*) vSeparatorNew, 0},
	{"windowMaximize", 1, (void*) windowMaximize, 0},
	{"theFileHasAValidUtf8Name", 1, (void*) theFileHasAValidUtf8Name, 0},
	{"splitStringIntoWords", 2, (void*) splitStringIntoWords, 0},
	{"windowCenter", 1, (void*) windowCenter, 0},
	{"windowMaximizeVertically", 1, (void*) windowMaximizeVertically, 0},
	{"dialogNew", 4, (void*) dialogNew, 0},
	{"miscSetPadding", 3, (void*) miscSetPadding, 0},
	{"miscSetAlignment", 3, (void*) miscSetAlignment, 0},
	{"sleepUntilEventIsRaised", 1, (void*) sleepUntilEventIsRaised, 0},
	{NULL, 0, NULL, 0} // terminating line
};
// END

#include<libgen.h> // for dirname. Cannot put it above because it imports basename(char*), where I need basename(const char *), from string.h



bool timeout_handler(){
	// REGION generate prolog message "timer"
	term_t gLatestEvent = PL_new_term_ref();
	PL_put_atom_chars(gLatestEvent, "timer");
	gLatestEvents.push_back(SegTerm(gLatestEvent));
	// END
	return true;
}


int main( int   argc,  char *argv[] ){
	string lCurrentDir = get_current_dir_name();
	gGtkMain = shared_ptr<Gtk::Main> (new Gtk::Main(&argc, &argv)); 
	// REGION initialize prolog
	PL_register_extensions(predicates);
	if (!PL_initialise(argc, argv))
		PL_halt(1);
	// END

// 	// REGION debug : testing function to split strings into words.
// 	ustring ls = "/usr/local/seguso ciao andr a Hber a?a 32 a32 cena perch mi va.pdf";
// 	std::set<ustring>lSplitResult =  gSplitStringIntoWords(ls);
// 	set<ustring>::const_iterator i;
// 	for(i=lSplitResult.begin();
// 	    i!=lSplitResult.end();
// 	    i++){
// 		cout << *i << endl;}
// 	// END

	// REGION extract path of where the file segusoLand.pl must be searched for
#ifndef LIBDIR
        // look for segusoLand.pl in the same dir as the executable
	string lPathWhereThePrologFileIsLocated = dirname(argv[0]);
#else
	string lPathWhereThePrologFileIsLocated = LIBDIR;
#endif
#ifndef IMAGEDIR
        // look for the img/ directory in the same dir as the executable
	string lPathWhereTheSubdirectoryImgIsLocated = dirname(argv[0]);
#else
	string lPathWhereTheSubdirectoryImgIsLocated = IMAGEDIR;
#endif
	//END



	// REGION initialize gtk tooltips. Wrap the Tooltips* into a shared_ptr so we needn't use delete.
	cout << "new Tooltips" << endl;
	gTooltips = shared_ptr<Tooltips>(new Tooltips());
	// END
	// REGION initialize GTK timer
	Glib::signal_timeout().connect(SigC::slot(&timeout_handler), 1000);
	// END
	//Gtk::Main::idle.connect(IdleFunc);
	// REGION compile and load the prolog program, contained in the file segusoLand.pl
	callPredicate("consult('" + lPathWhereThePrologFileIsLocated + "/segusoLand')");
	// END
	// REGION run the main program logic, i.e. call the prolog predicate run/1  (/1 means it takes one argument)
	callPredicate("run('" + lPathWhereTheSubdirectoryImgIsLocated + "')");
	// END
	// REGION clear all containers that contain SegTerms, before calling PL_halt, otherwise you get segmentation fault
	gLatestEvents.clear();
	gPrologNameToGtkWidget.clear();
	// END

	// REGION replace previous working directory, because the
	//        prolog predicate run/1 has changed it to
	//        lPathWhereTheSubdirectoryImgIsLocated
	chdir(lCurrentDir.c_str());
	// END

	PL_halt(1); // calls exit(1)
	// REGION 

}


// 	/// A Label with the capability of wrapping itself if its
// 	/// width exceeds P pixels.
// 	class MyLabel2 : public Label{
// 	public:
// 		MyLabel2(string aTextWithoutNewlines) : Label(aTextWithoutNewlines){
// 			mTextWithoutNewlines = aTextWithoutNewlines;
// 			SetWrapPixel(100);
// 		}
// 		// Adds newlines in the label, in order not to make it
// 		// be wider than P pixels. This must be called each
// 		// time the width of the widget cointaining the label
// 		// changes.
// 		//
// 		//  TOO SLOW. Also, it splits words, which is
// 		//  undesirable.
// 		void SetWrapPixel(int p){
// 			Glib::ustring lTextWithNewlines = "";
// 			Glib::ustring lPossibleTextWithNewlines = "";
// 			for (Glib::ustring::iterator curChar=mTextWithoutNewlines.begin(); 
// 			     curChar!=mTextWithoutNewlines.end(); 
// 			     curChar++){
// 				lPossibleTextWithNewlines += *curChar;
// 				if (pixelWidthOfText(lPossibleTextWithNewlines)<p)
// 					lTextWithNewlines = lPossibleTextWithNewlines;
// 				else{
// 					lTextWithNewlines +='\n';
// 					lTextWithNewlines += *curChar;
// 					lPossibleTextWithNewlines = lTextWithNewlines;
// 				}
// 			}
// 			this->set_text(lTextWithNewlines);
// 		}
// 	private:
// 		Glib::ustring mTextWithoutNewlines;
// 		int pixelWidthOfText(Glib::ustring aTextPossiblyWithNewlines){
// 			if (mLayout ==0)
// 				mLayout= create_pango_layout(aTextPossiblyWithNewlines);
// 			else
// 				mLayout->set_text(aTextPossiblyWithNewlines);
// 			int w,h;
// 			mLayout->get_pixel_size(w, h);
// 			return w;
// 		}
// 		/// optimization: store the layout as member, in order
// 		/// to avoid to create it each time
// 		Glib::RefPtr<Pango::Layout> mLayout; 
// 	};



	// 	/// A Label with the capability of wrapping itself if its
	// 	/// width exceeds P pixels.
	// 	//
	// 	// NOTE: this simple method inexplicably doesn't work. Anyway
	// 	// it would be too slow. So I must manually add newlines to
	// 	// the string.
	// 	//
	// 	// void SetWrapPixel(int p){
	// 	//     get_layout()->set_width(p*Pango::SCALE);
	// 	//     get_layout()->set_wrap(Pango::WRAP_WORD_CHAR); }
	// 	class MyLabel3 : public Label{
	// 	public:
	// 		MyLabel3(string aTextWithoutNewlines) : Label(){
	// 			mTextWithoutNewlines = aTextWithoutNewlines;
	// 			mCurrentWrapPixel = 120;
	// 			SetWrapPixel(60); // calls Label::set_text();
	// 		}
	// 		// SetWrapPixel: this function adds newlines to the
	// 		// label text, in order not to make it be wider than P
	// 		// pixels. Also it does not break words. Uses pango
	// 		// for the wrapping. This would be perfect but I can't
	// 		// write the function extract_text_of_line()!
	// 		void SetWrapPixel(int p);
	// 	protected:
	// 		virtual bool on_expose_event(GdkEventExpose* e){
	// 			cout << "expose "<< mTextWithoutNewlines <<endl;
	// 			Label::on_expose_event(e);

	// 		}
	// 	private:
	// 		Glib::ustring mTextWithoutNewlines;
	// 		int mCurrentWrapPixel;
	// 		Glib::ustring extract_text_of_line(Glib::RefPtr<Pango::LayoutLine> aLayoutLine, 
	// 						   Glib::ustring aText);
	// 	};
